=== modified file 'Makefile'
--- Makefile	2007-12-17 01:45:32 +0000
+++ Makefile	2008-04-28 03:52:09 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -198,3 +198,48 @@
 	$(PYTHON) tools/win32/ostools.py remove bzr-setup*.exe
 	$(PYTHON) tools/win32/ostools.py remove bzr-*win32.exe
 	$(PYTHON) tools/win32/ostools.py remove dist
+
+.PHONY: dist dist-upload-escudero check-dist-tarball
+
+# build a distribution tarball.
+#
+# this method of copying the pyrex generated files is a bit ugly; it would be
+# nicer to generate it from distutils.
+#
+# these are a bit ubuntu-specific.
+dist: 
+	version=`./bzr version --short` && \
+	echo Building distribution of bzr $$version && \
+	expbasedir=`mktemp -d` && \
+	expdir=$$expbasedir/bzr-$$version && \
+	tarball=$$PWD/../bzr-$$version.tar.gz && \
+	$(MAKE) clean && \
+	$(MAKE) && \
+	bzr export $$expdir && \
+	cp bzrlib/*.c $$expdir/bzrlib/. && \
+	tar cfz $$tarball -C $$expbasedir bzr-$$version && \
+	gpg --detach-sign $$tarball && \
+	echo $$tarball done.
+
+# run all tests in a previously built tarball
+check-dist-tarball:
+	tmpdir=`mktemp -d` && \
+	version=`./bzr version --short` && \
+	tarball=$$PWD/../bzr-$$version.tar.gz && \
+	tar Cxz $$tmpdir -f $$tarball && \
+	$(MAKE) -C $$tmpdir/bzr-$$version check 
+
+
+# upload previously built tarball to the download directory on bazaar-vcs.org,
+# and verify that it can be downloaded ok.
+dist-upload-escudero:
+	version=`./bzr version --short` && \
+	tarball=../bzr-$$version.tar.gz && \
+	scp $$tarball $$tarball.sig \
+	    escudero.ubuntu.com:/srv/bazaar.canonical.com/www/releases/src \
+		&& \
+	echo verifying over http... && \
+	curl http://bazaar-vcs.org/releases/src/bzr-$$version.tar.gz \
+		| diff -s - $$tarball && \
+	curl http://bazaar-vcs.org/releases/src/bzr-$$version.tar.gz.sig \
+		| diff -s - $$tarball.sig 

=== modified file 'NEWS'
--- NEWS	2008-04-27 22:38:34 +0000
+++ NEWS	2008-05-01 07:03:04 +0000
@@ -24,14 +24,41 @@
     * Diff now handles revision specs like "branch:" and "submit:" more
       efficiently.  (Aaron Bentley)
 
+    * Pull completes much faster when there is nothing to pull.
+      (Aaron Bentley)
+
   BUGFIXES:
 
     * Avoid muttering every time a child update does not cause a progress bar
       update. (John Arbash Meinel, #213771)
 
+    * ``Branch.reconcile()`` is now implemented. This allows ``bzr reconcile``
+      to fix when a Branch has a non-canonical mainline history. ``bzr check``
+      also detects this condition. (John Arbash Meinel, #177855)
+
     * ``bzr commit`` now works with Microsoft's FTP service.
       (Andreas Deininger)
 
+    * Conversion from non-rich-root to rich-root(-pack) updates inventory
+      sha1s, even when bundles are used.  (Aaron Bentley, #181391)
+
+    * Conversion from non-rich-root to rich-root(-pack) works correctly even
+      though search keys are not topologically sorted.  (Aaron Bentley)
+
+    * Conversion from non-rich-root to rich-root(-pack) works even when a
+      parent revision has a different root id.  (Aaron Bentley, #177874)
+
+    * Fetching all revisions from a repository does not cause pack collisions.
+      (Robert Collins, Aaron Bentley, #212908)
+
+    * Fix error about "attempt to add line-delta in non-delta knit".
+      (Andrew Bennetts, #217701)
+
+    * Pushing a branch in "dirstate" format (Branch5) over bzr+ssh would break
+      if the remote server was < version 1.2. This was due to a bug in the
+      RemoteRepository.get_parent_map() fallback code.
+      (John Arbash Meinel, #214894)
+
     * Set SO_REUSEADDR on server sockets of ``bzr serve`` to avoid problems
       rebinding the socket when starting the server a second time.
       (John Arbash Meinel, Martin Pool, #164288)
@@ -52,10 +79,21 @@
 
   TESTING:
 
+    * Add ``thread_name_suffix`` parameter to SmartTCPServer_for_testing, to
+      make it easy to identify which test spawned a thread with an unhandled
+      exception. (Andrew Bennetts)
+
   INTERNALS:
 
+    * ``Hooks.install_hook`` is now deprecated in favour of
+      ``Hooks.install_named_hook`` which adds a required ``name`` parameter, to
+      avoid having to call ``Hooks.name_hook``. (Daniel Watkins)
+
     * Implement xml8 serializer.  (Aaron Bentley)
 
+    * New form ``@deprecated_method(deprecated_in(1, 5, 0))`` for making 
+      deprecation wrappers.  (Martin Pool)
+
   API BREAKS:
 
     * Repository.get_data_stream, Repository.get_data_stream_for_search(),
@@ -64,6 +102,34 @@
       (Aaron Bentley)
 
 
+bzr 1.4rc2 2008-04-21
+---------------------
+
+  BUG FIXES:
+
+    * ``bzr log -r ..X bzr://`` was failing, because it was getting a request
+      for ``revision_id=None`` which was not a string.
+      (John Arbash Meinel, #211661)
+
+    * Fixed a bug in handling ghost revisions when logging changes in a 
+      particular file.  (John Arbash Meinel, #209948)
+
+    * Fix error about "attempt to add line-delta in non-delta knit".
+      (Andrew Bennetts, #205156)
+
+    * Fixed performance degradation in fetching from knit repositories to
+      knits and packs due to parsing the entire revisions.kndx on every graph
+      walk iteration fixed by using the Repository.get_graph API.  There was
+      another regression in knit => knit fetching which re-read the index for
+      every revision each side had in common.
+      (Robert Collins, John Arbash Meinel)
+
+
+bzr 1.4rc1 2008-04-11
+---------------------
+
+
+
 bzr 1.4rc1 2008-04-11
 ---------------------
 
@@ -146,8 +212,11 @@
       one is present.  You can use the new ``nosmart+`` transport decorator
       to get the old behaviour.  (Andrew Bennetts)
 
-     * Various operations with revision specs and commands that calculate
-       revnos and revision ids are faster.  (John A. Meinel, Aaron Bentley)
+    * The ``version`` command takes a ``--short`` option to print just the
+      version number, for easier use in scripts.  (Martin Pool)
+
+    * Various operations with revision specs and commands that calculate
+      revnos and revision ids are faster.  (John A. Meinel, Aaron Bentley)
 
   BUGFIXES:
 
@@ -324,6 +393,9 @@
       had already failed, and should not be relied upon by code. 
       (Martin Pool, #109520)
 
+    * ``make dist`` target to build a release tarball, and also 
+      ``check-dist-tarball`` and ``dist-upload-escudero``.  (Martin Pool)
+
     * The ``read_response_tuple`` method of ``SmartClientRequestProtocol*``
       classes will now raise ``UnknownSmartMethod`` when appropriate, so that
       callers don't need to try distinguish unknown request errors from other

=== modified file 'bzr'
--- bzr	2008-04-02 01:46:35 +0000
+++ bzr	2008-04-28 03:52:09 +0000
@@ -22,7 +22,7 @@
 import sys
 
 # update this on each release
-_script_version = (1, 4, 0)
+_script_version = (1, 5, 0)
 
 if __doc__ is None:
     print "bzr does not support python -OO."

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2008-04-02 01:46:35 +0000
+++ bzrlib/__init__.py	2008-04-30 08:04:11 +0000
@@ -41,13 +41,13 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (1, 4, 0, 'dev', 0)
+version_info = (1, 5, 0, 'dev', 0)
 
 # API compatibility version: bzrlib is currently API compatible with 0.18.
 api_minimum_version = (0, 18, 0)
 
 def _format_version_tuple(version_info):
-    """Turn a version number 5-tuple into a short string.
+    """Turn a version number 3-tuple or 5-tuple into a short string.
 
     This format matches <http://docs.python.org/dist/meta-data.html>
     and the typical presentation used in Python output.
@@ -61,11 +61,15 @@
     1.2dev
     >>> print _format_version_tuple((1, 1, 1, 'candidate', 2))
     1.1.1rc2
+    >>> print _format_version_tuple((1, 4, 0))
+    1.4
     """
     if version_info[2] == 0:
         main_version = '%d.%d' % version_info[:2]
     else:
         main_version = '%d.%d.%d' % version_info[:3]
+    if len(version_info) <= 3:
+        return main_version
 
     __release_type = version_info[3]
     __sub = version_info[4]

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2008-04-11 04:51:44 +0000
+++ bzrlib/branch.py	2008-04-30 13:32:15 +0000
@@ -704,7 +704,16 @@
         :return: A BranchCheckResult.
         """
         mainline_parent_id = None
-        for revision_id in self.revision_history():
+        last_revno, last_revision_id = self.last_revision_info()
+        real_rev_history = list(self.repository.iter_reverse_revision_history(
+                                last_revision_id))
+        real_rev_history.reverse()
+        if len(real_rev_history) != last_revno:
+            raise errors.BzrCheckError('revno does not match len(mainline)'
+                ' %s != %s' % (last_revno, len(real_rev_history)))
+        # TODO: We should probably also check that real_rev_history actually
+        #       matches self.revision_history()
+        for revision_id in real_rev_history:
             try:
                 revision = self.repository.get_revision(revision_id)
             except errors.NoSuchRevision, e:
@@ -783,6 +792,14 @@
             basis_tree.unlock()
         return tree
 
+    @needs_write_lock
+    def reconcile(self, thorough=True):
+        """Make sure the data stored in this branch is consistent."""
+        from bzrlib.reconcile import BranchReconciler
+        reconciler = BranchReconciler(self, thorough=thorough)
+        reconciler.reconcile()
+        return reconciler
+
     def reference_parent(self, file_id, path):
         """Return the parent branch for a tree-reference file_id
         :param file_id: The file_id of the tree reference

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2008-04-24 03:30:54 +0000
+++ bzrlib/builtins.py	2008-04-29 19:13:40 +0000
@@ -669,22 +669,28 @@
                 raise errors.BzrCommandError(
                     'bzr pull --revision takes one value.')
 
-        if verbose:
-            old_rh = branch_to.revision_history()
-        if tree_to is not None:
-            change_reporter = delta._ChangeReporter(
-                unversioned_filter=tree_to.is_ignored)
-            result = tree_to.pull(branch_from, overwrite, revision_id,
-                                  change_reporter,
-                                  possible_transports=possible_transports)
-        else:
-            result = branch_to.pull(branch_from, overwrite, revision_id)
+        branch_to.lock_write()
+        try:
+            if tree_to is not None:
+                change_reporter = delta._ChangeReporter(
+                    unversioned_filter=tree_to.is_ignored)
+                result = tree_to.pull(branch_from, overwrite, revision_id,
+                                      change_reporter,
+                                      possible_transports=possible_transports)
+            else:
+                result = branch_to.pull(branch_from, overwrite, revision_id)
 
-        result.report(self.outf)
-        if verbose:
-            new_rh = branch_to.revision_history()
-            log.show_changed_revisions(branch_to, old_rh, new_rh,
-                                       to_file=self.outf)
+            result.report(self.outf)
+            if verbose and result.old_revid != result.new_revid:
+                old_rh = list(
+                    branch_to.repository.iter_reverse_revision_history(
+                    result.old_revid))
+                old_rh.reverse()
+                new_rh = branch_to.revision_history()
+                log.show_changed_revisions(branch_to, old_rh, new_rh,
+                                           to_file=self.outf)
+        finally:
+            branch_to.unlock()
 
 
 class cmd_push(Command):
@@ -2719,11 +2725,17 @@
     """Show version of bzr."""
 
     encoding_type = 'replace'
+    takes_options = [
+        Option("short", help="Print just the version number."),
+        ]
 
     @display_command
-    def run(self):
+    def run(self, short=False):
         from bzrlib.version import show_version
-        show_version(to_file=self.outf)
+        if short:
+            self.outf.write(bzrlib.version_string + '\n')
+        else:
+            show_version(to_file=self.outf)
 
 
 class cmd_rocks(Command):
@@ -3659,8 +3671,9 @@
     specified revision.  For example, "bzr uncommit -r 15" will leave the
     branch at revision 15.
 
-    In the future, uncommit will create a revision bundle, which can then
-    be re-applied.
+    Uncommit leaves the working tree ready for a new commit.  The only change
+    it may make is to restore any pending merges that were present before
+    the commit.
     """
 
     # TODO: jam 20060108 Add an option to allow uncommit to remove

=== modified file 'bzrlib/bundle/bundle_data.py'
--- bzrlib/bundle/bundle_data.py	2007-09-25 08:14:12 +0000
+++ bzrlib/bundle/bundle_data.py	2008-04-28 00:47:10 +0000
@@ -260,21 +260,6 @@
             elif revision_id not in checked:
                 missing[revision_id] = sha1
 
-        for inv_id, sha1 in inv_to_sha.iteritems():
-            if repository.has_revision(inv_id):
-                # Note: branch.get_inventory_sha1() just returns the value that
-                # is stored in the revision text, and that value may be out
-                # of date. This is bogus, because that means we aren't
-                # validating the actual text, just that we wrote and read the
-                # string. But for now, what the hell.
-                local_sha1 = repository.get_inventory_sha1(inv_id)
-                if sha1 != local_sha1:
-                    raise BzrError('sha1 mismatch. For inventory id {%s}' 
-                                   'local: %s, bundle: %s' % 
-                                   (inv_id, local_sha1, sha1))
-                else:
-                    count += 1
-
         if len(missing) > 0:
             # I don't know if this is an error yet
             warning('Not all revision hashes could be validated.'

=== modified file 'bzrlib/bundle/serializer/v4.py'
--- bzrlib/bundle/serializer/v4.py	2008-03-19 04:39:04 +0000
+++ bzrlib/bundle/serializer/v4.py	2008-04-28 04:00:05 +0000
@@ -588,7 +588,11 @@
     def _install_revision(self, revision_id, metadata, text):
         if self._repository.has_revision(revision_id):
             return
-        self._repository._add_revision_text(revision_id, text)
+        if self._info['serializer'] == self._repository._serializer.format_num:
+            self._repository._add_revision_text(revision_id, text)
+        else:
+            revision = self._source_serializer.read_revision_from_string(text)
+            self._repository.add_revision(revision.revision_id, revision)
 
     def _install_signature(self, revision_id, metadata, text):
         transaction = self._repository.get_transaction()

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2008-04-26 06:07:28 +0000
+++ bzrlib/fetch.py	2008-04-30 04:51:16 +0000
@@ -158,7 +158,10 @@
         pb = bzrlib.ui.ui_factory.nested_progress_bar()
         try:
             revs = search.get_keys()
-            data_to_fetch = self.from_repository.item_keys_introduced_by(revs, pb)
+            graph = self.from_repository.get_graph()
+            revs = list(graph.iter_topo_order(revs))
+            data_to_fetch = self.from_repository.item_keys_introduced_by(revs,
+                                                                         pb)
             for knit_kind, file_id, revisions in data_to_fetch:
                 if knit_kind != phase:
                     phase = knit_kind
@@ -268,8 +271,7 @@
             except errors.NoSuchRevision:
                 # not signed.
                 pass
-            to_store.add_revision(self.from_repository.get_revision(rev),
-                                  to_txn)
+            self._copy_revision(rev, to_txn)
             count += 1
         # fixup inventory if needed: 
         # this is expensive because we have no inverse index to current ghosts.
@@ -277,7 +279,11 @@
         # so we just-do-it.
         # FIXME: repository should inform if this is needed.
         self.to_repository.reconcile()
-    
+
+    def _copy_revision(self, rev, to_txn):
+        to_store = self.to_repository._revision_store
+        to_store.add_revision(self.from_repository.get_revision(rev), to_txn)
+
 
 class KnitRepoFetcher(RepoFetcher):
     """This is a knit format repository specific fetcher.
@@ -296,11 +302,15 @@
         from_sf = self.from_repository._revision_store.get_signature_file(
             from_transaction)
         to_sf.join(from_sf, version_ids=revs, ignore_missing=True)
+        self._fetch_just_revision_texts(revs, from_transaction, to_transaction)
+
+    def _fetch_just_revision_texts(self, version_ids, from_transaction,
+                                   to_transaction):
         to_rf = self.to_repository._revision_store.get_revision_file(
             to_transaction)
         from_rf = self.from_repository._revision_store.get_revision_file(
             from_transaction)
-        to_rf.join(from_rf, version_ids=revs)
+        to_rf.join(from_rf, version_ids=version_ids)
 
 
 class Inter1and2Helper(object):
@@ -337,27 +347,54 @@
                 yield tree
             revs = revs[100:]
 
+    def _find_root_ids(self, revs, parent_map, graph):
+        revision_root = {}
+        planned_versions = {}
+        for tree in self.iter_rev_trees(revs):
+            revision_id = tree.inventory.root.revision
+            root_id = tree.get_root_id()
+            planned_versions.setdefault(root_id, []).append(revision_id)
+            revision_root[revision_id] = root_id
+        # Find out which parents we don't already know root ids for
+        parents = set()
+        for revision_parents in parent_map.itervalues():
+            parents.update(revision_parents)
+        parents.difference_update(revision_root.keys() + [NULL_REVISION])
+        # Limit to revisions present in the versionedfile
+        parents = graph.get_parent_map(parents).keys()
+        for tree in self.iter_rev_trees(parents):
+            root_id = tree.get_root_id()
+            revision_root[tree.get_revision_id()] = root_id
+        return revision_root, planned_versions
+
     def generate_root_texts(self, revs):
         """Generate VersionedFiles for all root ids.
-        
+
         :param revs: the revisions to include
         """
-        inventory_weave = self.source.get_inventory_weave()
-        parent_texts = {}
-        versionedfile = {}
         to_store = self.target.weave_store
-        parent_map = self.source.get_graph().get_parent_map(revs)
-        for tree in self.iter_rev_trees(revs):
-            revision_id = tree.inventory.root.revision
-            root_id = tree.get_root_id()
-            parents = parent_map[revision_id]
-            if parents[0] == NULL_REVISION:
-                parents = ()
-            if root_id not in versionedfile:
-                versionedfile[root_id] = to_store.get_weave_or_empty(root_id,
-                    self.target.get_transaction())
-            _, _, parent_texts[root_id] = versionedfile[root_id].add_lines(
-                revision_id, parents, [], parent_texts)
+        graph = self.source.get_graph()
+        parent_map = graph.get_parent_map(revs)
+        revision_root, planned_versions = self._find_root_ids(
+            revs, parent_map, graph)
+        for root_id, versions in planned_versions.iteritems():
+            versionedfile = to_store.get_weave_or_empty(root_id,
+                self.target.get_transaction())
+            parent_texts = {}
+            for revision_id in versions:
+                if revision_id in versionedfile:
+                    continue
+                parents = parent_map[revision_id]
+                # We drop revision parents with different file-ids, because
+                # a version cannot have a version with another file-id as its
+                # parent.
+                # When a parent revision is a ghost, we guess that its root id
+                # was unchanged.
+                parents = tuple(p for p in parents if p != NULL_REVISION
+                    and revision_root.get(p, root_id) == root_id)
+                result = versionedfile.add_lines_with_ghosts(
+                    revision_id, parents, [], parent_texts)
+                parent_texts[revision_id] = result[2]
 
     def regenerate_inventory(self, revs):
         """Generate a new inventory versionedfile in target, convertin data.
@@ -371,6 +408,10 @@
             self.target.add_inventory(tree.get_revision_id(), tree.inventory,
                                       parents)
 
+    def fetch_revisions(self, revision_ids):
+        for revision in self.source.get_revisions(revision_ids):
+            self.target.add_revision(revision.revision_id, revision)
+
 
 class Model1toKnit2Fetcher(GenericRepoFetcher):
     """Fetch from a Model1 repository into a Knit2 repository
@@ -386,7 +427,10 @@
 
     def _fetch_inventory_weave(self, revs, pb):
         self.helper.regenerate_inventory(revs)
- 
+
+    def _copy_revision(self, rev, to_txn):
+        self.helper.fetch_revisions([rev])
+
 
 class Knit1to2Fetcher(KnitRepoFetcher):
     """Fetch from a Knit1 repository into a Knit2 repository"""
@@ -403,6 +447,10 @@
     def _fetch_inventory_weave(self, revs, pb):
         self.helper.regenerate_inventory(revs)
 
+    def _fetch_just_revision_texts(self, version_ids, from_transaction,
+                                   to_transaction):
+        self.helper.fetch_revisions(version_ids)
+
 
 class RemoteToOtherFetcher(GenericRepoFetcher):
 

=== modified file 'bzrlib/hooks.py'
--- bzrlib/hooks.py	2007-06-28 06:32:03 +0000
+++ bzrlib/hooks.py	2008-04-30 13:21:37 +0000
@@ -17,6 +17,7 @@
 
 """Support for plugin hooking logic."""
 from bzrlib.lazy_import import lazy_import
+from bzrlib.symbol_versioning import deprecated_method, one_five
 lazy_import(globals(), """
 from bzrlib import (
         errors,
@@ -45,6 +46,7 @@
         """
         return self._callable_names.get(a_callable, "No hook name")
 
+    @deprecated_method(one_five)
     def install_hook(self, hook_name, a_callable):
         """Install a_callable in to the hook hook_name.
 
@@ -54,10 +56,25 @@
             The exact signature will depend on the hook - see the __init__ 
             method of BranchHooks for details on each hook.
         """
+        self.install_named_hook(hook_name, a_callable, None)
+
+    def install_named_hook(self, hook_name, a_callable, name):
+        """Install a_callable in to the hook hook_name, and label it name.
+
+        :param hook_name: A hook name. See the __init__ method of BranchHooks
+            for the complete list of hooks.
+        :param a_callable: The callable to be invoked when the hook triggers.
+            The exact signature will depend on the hook - see the __init__ 
+            method of BranchHooks for details on each hook.
+        :param name: A name to associate a_callable with, to show users what is
+            running.
+        """
         try:
             self[hook_name].append(a_callable)
         except KeyError:
             raise errors.UnknownHook(self.__class__.__name__, hook_name)
+        if name is not None:
+            self.name_hook(a_callable, name)
 
     def name_hook(self, a_callable, name):
         """Associate name with a_callable to show users what is running."""

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2008-04-09 23:35:55 +0000
+++ bzrlib/knit.py	2008-04-28 06:24:16 +0000
@@ -843,10 +843,28 @@
                             'on the source repository, and "bzr reconcile" '
                             'if necessary.' %
                             (version_id, parents[0]))
+                    if not self.delta:
+                        # We received a line-delta record for a non-delta knit.
+                        # Convert it to a fulltext.
+                        gzip_bytes = reader_callable(length)
+                        self._convert_line_delta_to_fulltext(
+                            gzip_bytes, version_id, parents)
+                        continue
+
                 self._add_raw_records(
                     [(version_id, options, parents, length)],
                     reader_callable(length))
 
+    def _convert_line_delta_to_fulltext(self, gzip_bytes, version_id, parents):
+        lines, sha1 = self._data._parse_record(version_id, gzip_bytes)
+        delta = self.factory.parse_line_delta(lines, version_id)
+        content = self.factory.make(self.get_lines(parents[0]), parents[0])
+        content.apply_delta(delta, version_id)
+        digest, len, content = self.add_lines(
+            version_id, parents, content.text())
+        if digest != sha1:
+            raise errors.VersionedFileInvalidChecksum(version_id)
+
     def _knit_from_datastream(self, (format, data_list, reader_callable)):
         """Create a knit object from a data stream.
 

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2008-04-18 17:54:34 +0000
+++ bzrlib/log.py	2008-04-28 03:52:09 +0000
@@ -464,8 +464,10 @@
     # This asks for all mainline revisions, which means we only have to spider
     # sideways, rather than depth history. That said, its still size-of-history
     # and should be addressed.
+    # mainline_revisions always includes an extra revision at the beginning, so
+    # don't request it.
     parent_map = dict(((key, value) for key, value in
-        graph.iter_ancestry(mainline_revisions) if value is not None))
+        graph.iter_ancestry(mainline_revisions[1:]) if value is not None))
     sorted_rev_list = topo_sort(parent_map.items())
     ancestry = {}
     for rev in sorted_rev_list:

=== modified file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	2008-04-05 13:45:37 +0000
+++ bzrlib/mutabletree.py	2008-05-01 03:08:59 +0000
@@ -240,6 +240,19 @@
         """
         raise NotImplementedError(self.mkdir)
 
+    @needs_write_lock
+    def put_file_bytes_non_atomic(self, file_id, bytes):
+        """Update the content of a file in the tree.
+        
+        Note that the file is written in-place rather than being
+        written to a temporary location and renamed. As a consequence,
+        readers can potentially see the file half-written.
+
+        :param file_id: file-id of the file
+        :param bytes: the new file contents
+        """
+        raise NotImplementedError(self.put_file_bytes_non_atomic)
+
     def set_parent_ids(self, revision_ids, allow_leftmost_as_ghost=False):
         """Set the parents ids of the working tree.
 

=== modified file 'bzrlib/reconcile.py'
--- bzrlib/reconcile.py	2008-03-26 21:42:35 +0000
+++ bzrlib/reconcile.py	2008-04-30 13:32:15 +0000
@@ -67,6 +67,9 @@
                               ancestry was being reported incorrectly.
         garbage_inventories: The number of inventory objects without revisions
                              that were garbage collected.
+        fixed_branch_history: None if there was no branch, False if the branch
+                              history was correct, True if the branch history
+                              needed to be re-normalized.
         """
         self.pb = ui.ui_factory.nested_progress_bar()
         try:
@@ -76,6 +79,22 @@
 
     def _reconcile(self):
         """Helper function for performing reconciliation."""
+        self._reconcile_branch()
+        self._reconcile_repository()
+
+    def _reconcile_branch(self):
+        try:
+            self.branch = self.bzrdir.open_branch()
+        except errors.NotBranchError:
+            # Nothing to check here
+            self.fixed_branch_history = None
+            return
+        self.pb.note('Reconciling branch %s',
+                     self.branch.base)
+        branch_reconciler = self.branch.reconcile(thorough=True)
+        self.fixed_branch_history = branch_reconciler.fixed_history
+
+    def _reconcile_repository(self):
         self.repo = self.bzrdir.find_repository()
         self.pb.note('Reconciling repository %s',
                      self.repo.bzrdir.root_transport.base)
@@ -92,6 +111,49 @@
             self.pb.note('Reconciliation complete.')
 
 
+class BranchReconciler(object):
+    """Reconciler that works on a branch."""
+
+    def __init__(self, a_branch, thorough=False):
+        self.fixed_history = None
+        self.thorough = thorough
+        self.branch = a_branch
+
+    def reconcile(self):
+        self.branch.lock_write()
+        try:
+            self.pb = ui.ui_factory.nested_progress_bar()
+            try:
+                self._reconcile_steps()
+            finally:
+                self.pb.finished()
+        finally:
+            self.branch.unlock()
+
+    def _reconcile_steps(self):
+        self._reconcile_revision_history()
+
+    def _reconcile_revision_history(self):
+        repo = self.branch.repository
+        last_revno, last_revision_id = self.branch.last_revision_info()
+        real_history = list(repo.iter_reverse_revision_history(
+                                last_revision_id))
+        real_history.reverse()
+        if last_revno != len(real_history):
+            self.fixed_history = True
+            # Technically for Branch5 formats, it is more efficient to use
+            # set_revision_history, as this will regenerate it again.
+            # Not really worth a whole BranchReconciler class just for this,
+            # though.
+            self.pb.note('Fixing last revision info %s => %s',
+                         last_revno, len(real_history))
+            self.branch.set_last_revision_info(len(real_history),
+                                               last_revision_id)
+        else:
+            self.fixed_history = False
+            self.pb.note('revision_history ok.')
+
+
 class RepoReconciler(object):
     """Reconciler that reconciles a repository.
 

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2008-04-10 16:41:50 +0000
+++ bzrlib/remote.py	2008-04-29 20:02:28 +0000
@@ -823,7 +823,21 @@
             # :- its because we're working with a deprecated server anyway, and
             # the user will almost certainly have seen a warning about the
             # server version already.
-            return self.get_revision_graph()
+            rg = self.get_revision_graph()
+            # There is an api discrepency between get_parent_map and
+            # get_revision_graph. Specifically, a "key:()" pair in
+            # get_revision_graph just means a node has no parents. For
+            # "get_parent_map" it means the node is a ghost. So fix up the
+            # graph to correct this.
+            #   https://bugs.launchpad.net/bzr/+bug/214894
+            # There is one other "bug" which is that ghosts in
+            # get_revision_graph() are not returned at all. But we won't worry
+            # about that for now.
+            for node_id, parent_ids in rg.iteritems():
+                if parent_ids == ():
+                    rg[node_id] = (NULL_REVISION,)
+            rg[NULL_REVISION] = ()
+            return rg
 
         keys = set(keys)
         if NULL_REVISION in keys:
@@ -859,7 +873,9 @@
         body = self._serialise_search_recipe(recipe)
         path = self.bzrdir._path_for_remote_call(self._client)
         for key in keys:
-            assert type(key) is str
+            if type(key) is not str:
+                raise ValueError(
+                    "key %r not a plain string" % (key,))
         verb = 'Repository.get_parent_map'
         args = (path,) + tuple(keys)
         try:

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2008-04-26 16:34:47 +0000
+++ bzrlib/repository.py	2008-04-30 06:54:40 +0000
@@ -542,14 +542,17 @@
             plaintext = Testament(rev, inv).as_short_text()
             self.store_revision_signature(
                 gpg.GPGStrategy(config), plaintext, revision_id)
-        if not revision_id in self.get_inventory_weave():
+        inventory_vf = self.get_inventory_weave()
+        if not revision_id in inventory_vf:
             if inv is None:
                 raise errors.WeaveRevisionNotPresent(revision_id,
-                                                     self.get_inventory_weave())
+                                                     inventory_vf)
             else:
                 # yes, this is not suitable for adding with ghosts.
-                rev.inventory_sha1 = self.add_inventory(revision_id, inv, 
+                rev.inventory_sha1 = self.add_inventory(revision_id, inv,
                                                         rev.parent_ids)
+        else:
+            rev.inventory_sha1 = inventory_vf.get_sha1s([revision_id])[0]
         self._revision_store.add_revision(rev, self.get_transaction())
 
     def _add_revision_text(self, revision_id, text):
@@ -2380,12 +2383,14 @@
         :param revision_ids: The start point for the search.
         :return: A set of revision ids.
         """
-        graph = self.source.get_graph()
         target_graph = self.target.get_graph()
+        revision_ids = frozenset(revision_ids)
+        if set(target_graph.get_parent_map(revision_ids)) == revision_ids:
+            return graph.SearchResult(revision_ids, set(), 0, set())
         missing_revs = set()
+        source_graph = self.source.get_graph()
         # ensure we don't pay silly lookup costs.
-        revision_ids = frozenset(revision_ids)
-        searcher = graph._make_breadth_first_searcher(revision_ids)
+        searcher = source_graph._make_breadth_first_searcher(revision_ids)
         null_set = frozenset([_mod_revision.NULL_REVISION])
         while True:
             try:
@@ -2742,6 +2747,11 @@
             # inventory parsing etc, IFF nothing to be copied is in the target.
             # till then:
             revision_ids = self.source.all_revision_ids()
+            revision_keys = [(revid,) for revid in revision_ids]
+            index = self.target._pack_collection.revision_index.combined_index
+            present_revision_ids = set(item[1][0] for item in
+                index.iter_entries(revision_keys))
+            revision_ids = set(revision_ids) - present_revision_ids
             # implementing the TODO will involve:
             # - detecting when all of a pack is selected
             # - avoiding as much as possible pre-selection, so the
@@ -2758,6 +2768,8 @@
                     find_ghosts=find_ghosts).get_keys()
             except errors.NoSuchRevision:
                 raise errors.InstallFailed([revision_id])
+            if len(revision_ids) == 0:
+                return (0, [])
         packs = self.source._pack_collection.all_packs()
         pack = Packer(self.target._pack_collection, packs, '.fetch',
             revision_ids).pack()

=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2008-04-09 00:34:54 +0000
+++ bzrlib/revisiontree.py	2008-04-28 16:37:29 +0000
@@ -96,6 +96,7 @@
         return w.annotate(self.inventory[file_id].revision)
 
     def get_file_size(self, file_id):
+        """See Tree.get_file_size"""
         return self._inventory[file_id].text_size
 
     def get_file_sha1(self, file_id, path=None, stat_value=None):

=== modified file 'bzrlib/smart/server.py'
--- bzrlib/smart/server.py	2008-04-21 01:07:17 +0000
+++ bzrlib/smart/server.py	2008-04-29 13:43:40 +0000
@@ -74,7 +74,7 @@
         self._stopped = threading.Event()
         self.root_client_path = root_client_path
 
-    def serve(self):
+    def serve(self, thread_name_suffix=''):
         self._should_terminate = False
         # for hooks we are letting code know that a server has started (and
         # later stopped).
@@ -116,7 +116,7 @@
                         if e.args[0] != errno.EBADF:
                             trace.warning("listening socket error: %s", e)
                     else:
-                        self.serve_conn(conn)
+                        self.serve_conn(conn, thread_name_suffix)
             except KeyboardInterrupt:
                 # dont log when CTRL-C'd.
                 raise
@@ -139,21 +139,23 @@
         """Return the url of the server"""
         return "bzr://%s:%d/" % self._sockname
 
-    def serve_conn(self, conn):
+    def serve_conn(self, conn, thread_name_suffix):
         # For WIN32, where the timeout value from the listening socket
         # propogates to the newly accepted socket.
         conn.setblocking(True)
         conn.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
         handler = SmartServerSocketStreamMedium(
             conn, self.backing_transport, self.root_client_path)
-        connection_thread = threading.Thread(None, handler.serve, name='smart-server-child')
+        thread_name = 'smart-server-child' + thread_name_suffix
+        connection_thread = threading.Thread(
+            None, handler.serve, name=thread_name)
         connection_thread.setDaemon(True)
         connection_thread.start()
 
-    def start_background_thread(self):
+    def start_background_thread(self, thread_name_suffix=''):
         self._started.clear()
         self._server_thread = threading.Thread(None,
-                self.serve,
+                self.serve, args=(thread_name_suffix,),
                 name='server-' + self.get_url())
         self._server_thread.setDaemon(True)
         self._server_thread.start()
@@ -213,9 +215,10 @@
     This server is backed by the process's cwd.
     """
 
-    def __init__(self):
+    def __init__(self, thread_name_suffix=''):
         SmartTCPServer.__init__(self, None)
         self.client_path_extra = None
+        self.thread_name_suffix = thread_name_suffix
         
     def get_backing_transport(self, backing_transport_server):
         """Get a backing transport from a server we are decorating."""
@@ -246,7 +249,7 @@
         self.backing_transport = transport.get_transport(
             self.chroot_server.get_url())
         self.root_client_path = self.client_path_extra = client_path_extra
-        self.start_background_thread()
+        self.start_background_thread(self.thread_name_suffix)
 
     def tearDown(self):
         self.stop_background_thread()

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2008-04-03 02:22:55 +0000
+++ bzrlib/status.py	2008-05-01 03:15:12 +0000
@@ -60,15 +60,15 @@
     :param show_ids: If set, includes each file's id.
     :param to_file: If set, write to this file (default stdout.)
     :param show_pending: If set, write pending merges.
-    :param revision: If None the compare latest revision with working tree
-        If not None it must be a RevisionSpec list.
-        If one revision show compared it with working tree.
-        If two revisions show status between first and second.
+    :param revision: If None, compare latest revision with working tree
+        If not None, it must be a RevisionSpec list.
+        If one revision, compare with working tree.
+        If two revisions, show status between first and second.
     :param short: If True, gives short SVN-style status lines.
     :param versioned: If True, only shows versioned files.
     """
     if show_unchanged is not None:
-        warn("show_status_trees with show_unchanged has been deprecated "
+        warn("show_tree_status with show_unchanged has been deprecated "
              "since bzrlib 0.9", DeprecationWarning, stacklevel=2)
 
     if to_file is None:

=== modified file 'bzrlib/store/revision/knit.py'
--- bzrlib/store/revision/knit.py	2008-04-04 00:06:58 +0000
+++ bzrlib/store/revision/knit.py	2008-04-28 06:24:16 +0000
@@ -113,7 +113,11 @@
 
     def get_revision_file(self, transaction):
         """Get the revision versioned file object."""
-        return self.versioned_file_store.get_weave_or_empty('revisions', transaction)
+        vf = self.versioned_file_store.get_weave_or_empty('revisions', transaction)
+        # The revisions knit should always be non-delta, so force delta=False
+        # here.
+        vf.delta = False
+        return vf
 
     def get_signature_file(self, transaction):
         """Get the signature text versioned file object."""

=== modified file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	2008-03-20 00:43:25 +0000
+++ bzrlib/symbol_versioning.py	2008-05-01 07:03:04 +0000
@@ -21,6 +21,7 @@
 """
 
 __all__ = ['deprecated_function',
+           'deprecated_in',
            'deprecated_list',
            'deprecated_method',
            'DEPRECATED_PARAMETER',
@@ -46,10 +47,13 @@
            'one_two',
            'one_three',
            'one_four',
+           'one_five',
            ]
 
 from warnings import warn
 
+import bzrlib
+
 
 DEPRECATED_PARAMETER = "A deprecated parameter marker."
 zero_seven = "%s was deprecated in version 0.7."
@@ -73,6 +77,17 @@
 one_two = "%s was deprecated in version 1.2."
 one_three = "%s was deprecated in version 1.3."
 one_four = "%s was deprecated in version 1.4."
+one_five = "%s was deprecated in version 1.5."
+
+
+def deprecated_in(version_tuple):
+    """Generate a message that something was deprecated in a release.
+
+    >>> deprecated_in((1, 4, 0))
+    '%s was deprecated in version 1.4'
+    """
+    return ("%s was deprecated in version "
+            + bzrlib._format_version_tuple(version_tuple))
 
 def set_warning_method(method):
     """Set the warning method to be used by this module.

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2008-04-07 04:44:56 +0000
+++ bzrlib/tests/__init__.py	2008-04-30 08:05:17 +0000
@@ -2839,6 +2839,7 @@
         'bzrlib.merge3',
         'bzrlib.option',
         'bzrlib.store',
+        'bzrlib.symbol_versioning',
         'bzrlib.tests',
         'bzrlib.timestamp',
         'bzrlib.version_info_formats.format_custom',

=== modified file 'bzrlib/tests/blackbox/test_hooks.py'
--- bzrlib/tests/blackbox/test_hooks.py	2008-03-08 18:25:44 +0000
+++ bzrlib/tests/blackbox/test_hooks.py	2008-04-23 09:46:52 +0000
@@ -37,7 +37,7 @@
     def test_hooks_with_unnamed_hook(self):
         self.make_branch('.')
         def foo(): return
-        Branch.hooks.install_hook('set_rh', foo)
+        Branch.hooks.install_named_hook('set_rh', foo, None)
         out, err = self.run_bzr('hooks')
         self._check_hooks_output(out, {'set_rh': ["No hook name"]})
 
@@ -45,8 +45,7 @@
         self.make_branch('.')
         def foo(): return
         name = "Foo Bar Hook"
-        Branch.hooks.install_hook('set_rh', foo)
-        Branch.hooks.name_hook(foo, name)
+        Branch.hooks.install_named_hook('set_rh', foo, name)
         out, err = self.run_bzr('hooks')
         self._check_hooks_output(out, {'set_rh': [name]})
 

=== modified file 'bzrlib/tests/blackbox/test_reconcile.py'
--- bzrlib/tests/blackbox/test_reconcile.py	2007-11-22 02:09:49 +0000
+++ bzrlib/tests/blackbox/test_reconcile.py	2008-04-30 15:02:49 +0000
@@ -46,10 +46,13 @@
             does_backup_text = "Inventory ok.\n"
         else:
             does_backup_text = ""
-        self.assertEqualDiff(out, "Reconciling repository %s\n"
+        self.assertEqualDiff(out, "Reconciling branch %s\n"
+                                  "revision_history ok.\n"
+                                  "Reconciling repository %s\n"
                                   "%s"
                                   "Reconciliation complete.\n" %
-                                  (t.bzrdir.root_transport.base,
+                                  (t.branch.base,
+                                   t.bzrdir.root_transport.base,
                                    does_backup_text))
         self.assertEqualDiff(err, "")
 
@@ -71,9 +74,13 @@
                 "Inventory regenerated.\n")
         else:
             does_backup_text = ""
-        expected = ("Reconciling repository %s\n"
+        expected = ("Reconciling branch %s\n"
+                    "revision_history ok.\n"
+                    "Reconciling repository %s\n"
                     "%s"
                     "Reconciliation complete.\n" %
-                    (t.bzrdir.root_transport.base, does_backup_text))
+                    (t.branch.base,
+                     t.bzrdir.root_transport.base,
+                     does_backup_text))
         self.assertEqualDiff(expected, out)
         self.assertEqualDiff(err, "")

=== modified file 'bzrlib/tests/blackbox/test_version.py'
--- bzrlib/tests/blackbox/test_version.py	2008-03-20 06:29:00 +0000
+++ bzrlib/tests/blackbox/test_version.py	2008-04-10 07:02:31 +0000
@@ -44,6 +44,10 @@
         self.assertContainsRe(out, r"(?m)^  Bazaar configuration:")
         self.assertContainsRe(out, r'(?m)^  Bazaar log file:.*\.bzr\.log')
 
+    def test_version_short(self):
+        out = self.run_bzr(["version", "--short"])[0]
+        self.assertEqualDiff(out, bzrlib.version_string + '\n')
+
 
 class TestVersionUnicodeOutput(TestCaseInTempDir):
 

=== modified file 'bzrlib/tests/branch_implementations/__init__.py'
--- bzrlib/tests/branch_implementations/__init__.py	2007-06-28 05:19:04 +0000
+++ bzrlib/tests/branch_implementations/__init__.py	2008-04-29 16:31:58 +0000
@@ -141,6 +141,7 @@
         'bzrlib.tests.branch_implementations.test_bound_sftp',
         'bzrlib.tests.branch_implementations.test_branch',
         'bzrlib.tests.branch_implementations.test_break_lock',
+        'bzrlib.tests.branch_implementations.test_check',
         'bzrlib.tests.branch_implementations.test_create_checkout',
         'bzrlib.tests.branch_implementations.test_commit',
         'bzrlib.tests.branch_implementations.test_get_revision_id_to_revno_map',
@@ -152,6 +153,7 @@
         'bzrlib.tests.branch_implementations.test_permissions',
         'bzrlib.tests.branch_implementations.test_pull',
         'bzrlib.tests.branch_implementations.test_push',
+        'bzrlib.tests.branch_implementations.test_reconcile',
         'bzrlib.tests.branch_implementations.test_revision_history',
         'bzrlib.tests.branch_implementations.test_revision_id_to_revno',
         'bzrlib.tests.branch_implementations.test_sprout',

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2008-03-28 04:58:53 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2008-04-29 15:47:14 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -373,14 +373,6 @@
         text = tree.branch._format.get_format_description()
         self.failUnless(len(text))
 
-    def test_check_branch_report_results(self):
-        """Checking a branch produces results which can be printed"""
-        branch = self.make_branch('.')
-        result = branch.check()
-        # reports results through logging
-        result.report_results(verbose=True)
-        result.report_results(verbose=False)
-
     def test_get_commit_builder(self):
         branch = self.make_branch(".")
         branch.lock_write()

=== added file 'bzrlib/tests/branch_implementations/test_check.py'
--- bzrlib/tests/branch_implementations/test_check.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_check.py	2008-04-29 17:01:24 +0000
@@ -0,0 +1,65 @@
+# Copyright (C) 2008 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for branch implementations - test check() functionality"""
+
+from bzrlib import errors
+from bzrlib.tests.branch_implementations import TestCaseWithBranch
+
+
+class TestBranchCheck(TestCaseWithBranch):
+
+    def test_check_detects_invalid_revhistory(self):
+        # Different formats have different ways of handling invalid revision
+        # histories, so the setup portion is customized
+        tree = self.make_branch_and_tree('test')
+        r1 = tree.commit('one')
+        r2 = tree.commit('two')
+        r3 = tree.commit('three')
+        r4 = tree.commit('four')
+        # create an alternate branch
+        tree.set_parent_ids([r1])
+        tree.branch.set_last_revision_info(1, r1)
+        r2b = tree.commit('two-b')
+
+        # now go back and merge the commit
+        tree.set_parent_ids([r4, r2b])
+        tree.branch.set_last_revision_info(4, r4)
+
+        r5 = tree.commit('five')
+        # Now, try to set an invalid history
+        try:
+            tree.branch.set_revision_history([r1, r2b, r5])
+        except errors.NotLefthandHistory:
+            # Branch5 allows set_revision_history to be wrong
+            # Branch6 raises NotLefthandHistory, but we can force bogus stuff
+            # with set_last_revision_info
+            tree.branch.set_last_revision_info(3, r5)
+
+        e = self.assertRaises(errors.BzrCheckError,
+                              tree.branch.check)
+        self.assertEqual('Internal check failed:'
+                         ' revno does not match len(mainline) 3 != 5', str(e))
+
+    def test_check_branch_report_results(self):
+        """Checking a branch produces results which can be printed"""
+        branch = self.make_branch('.')
+        result = branch.check()
+        # reports results through logging
+        result.report_results(verbose=True)
+        result.report_results(verbose=False)
+
+

=== modified file 'bzrlib/tests/branch_implementations/test_commit.py'
--- bzrlib/tests/branch_implementations/test_commit.py	2008-02-25 01:38:31 +0000
+++ bzrlib/tests/branch_implementations/test_commit.py	2008-03-10 13:40:38 +0000
@@ -68,8 +68,8 @@
 
     def test_post_commit_to_origin(self):
         tree = self.make_branch_and_memory_tree('branch')
-        Branch.hooks.install_hook('post_commit',
-            self.capture_post_commit_hook)
+        Branch.hooks.install_named_hook('post_commit',
+            self.capture_post_commit_hook, None)
         tree.lock_write()
         tree.add('')
         revid = tree.commit('a revision')
@@ -90,8 +90,8 @@
         except errors.UpgradeRequired:
             # cant bind this format, the test is irrelevant.
             return
-        Branch.hooks.install_hook('post_commit',
-            self.capture_post_commit_hook)
+        Branch.hooks.install_named_hook('post_commit',
+            self.capture_post_commit_hook, None)
         tree.lock_write()
         tree.add('')
         revid = tree.commit('a revision')
@@ -108,8 +108,8 @@
         tree.lock_write()
         tree.add('')
         revid = tree.commit('first revision')
-        Branch.hooks.install_hook('post_commit',
-            self.capture_post_commit_hook)
+        Branch.hooks.install_named_hook('post_commit',
+            self.capture_post_commit_hook, None)
         revid2 = tree.commit('second revision')
         # having committed from up the branch, we should get the
         # before and after revnos and revids correctly.
@@ -127,7 +127,8 @@
         tree.lock_write()
         tree.add('')
         root_delta.added = [('', tree.path2id(''), 'directory')]
-        Branch.hooks.install_hook("pre_commit", self.capture_pre_commit_hook)
+        Branch.hooks.install_named_hook("pre_commit",
+            self.capture_pre_commit_hook, None)
         revid1 = tree.commit('first revision')
         revid2 = tree.commit('second revision')
         self.assertEqual([
@@ -149,8 +150,9 @@
                       old_revno, old_revid, new_revno, new_revid,
                       tree_delta, future_tree):
             raise PreCommitException(new_revid)
-        Branch.hooks.install_hook("pre_commit", self.capture_pre_commit_hook)
-        Branch.hooks.install_hook("pre_commit", hook_func)
+        Branch.hooks.install_named_hook("pre_commit",
+            self.capture_pre_commit_hook, None)
+        Branch.hooks.install_named_hook("pre_commit", hook_func, None)
         revids = [None, None, None]
         # this commit will raise an exception
         # so the commit is rolled back and revno unchanged
@@ -160,7 +162,8 @@
         # unregister all pre_commit hooks
         Branch.hooks["pre_commit"] = []
         # and re-register the capture hook
-        Branch.hooks.install_hook("pre_commit", self.capture_pre_commit_hook)
+        Branch.hooks.install_named_hook("pre_commit",
+            self.capture_pre_commit_hook, None)
         # now these commits should go through
         for i in range(1, 3):
             revids[i] = tree.commit('message')
@@ -199,7 +202,8 @@
             tree.unversion(['to_be_unversioned_id'])
             tree.mkdir('added_dir', 'added_dir_id')
             # start to capture pre_commit delta
-            Branch.hooks.install_hook("pre_commit", self.capture_pre_commit_hook)
+            Branch.hooks.install_named_hook("pre_commit",
+                self.capture_pre_commit_hook, None)
             revid2 = tree.commit('second revision')
         finally:
             tree.unlock()

=== modified file 'bzrlib/tests/branch_implementations/test_hooks.py'
--- bzrlib/tests/branch_implementations/test_hooks.py	2008-04-09 06:43:42 +0000
+++ bzrlib/tests/branch_implementations/test_hooks.py	2008-04-20 08:21:39 +0000
@@ -37,7 +37,8 @@
 
     def test_set_rh_empty_history(self):
         branch = self.make_branch('source')
-        Branch.hooks.install_hook('set_rh', self.capture_set_rh_hook)
+        Branch.hooks.install_named_hook('set_rh', self.capture_set_rh_hook,
+                                        None)
         branch.set_revision_history([])
         self.assertEqual(self.hook_calls,
             [('set_rh', branch, [], True)])
@@ -50,7 +51,8 @@
         tree.commit('empty commit', rev_id='foo')
         tree.unlock()
         branch = tree.branch
-        Branch.hooks.install_hook('set_rh', self.capture_set_rh_hook)
+        Branch.hooks.install_named_hook('set_rh', self.capture_set_rh_hook,
+                                        None)
         # some branches require that their history be set to a revision in the
         # repository
         branch.set_revision_history(['f\xc2\xb5'])
@@ -59,15 +61,18 @@
 
     def test_set_rh_branch_is_locked(self):
         branch = self.make_branch('source')
-        Branch.hooks.install_hook('set_rh', self.capture_set_rh_hook)
+        Branch.hooks.install_named_hook('set_rh', self.capture_set_rh_hook,
+                                        None)
         branch.set_revision_history([])
         self.assertEqual(self.hook_calls,
             [('set_rh', branch, [], True)])
 
     def test_set_rh_calls_all_hooks_no_errors(self):
         branch = self.make_branch('source')
-        Branch.hooks.install_hook('set_rh', self.capture_set_rh_hook)
-        Branch.hooks.install_hook('set_rh', self.capture_set_rh_hook)
+        Branch.hooks.install_named_hook('set_rh', self.capture_set_rh_hook,
+                                        None)
+        Branch.hooks.install_named_hook('set_rh', self.capture_set_rh_hook,
+                                        None)
         branch.set_revision_history([])
         self.assertEqual(self.hook_calls,
             [('set_rh', branch, [], True),
@@ -92,8 +97,10 @@
 
     def test_post_change_branch_tip_empty_history(self):
         branch = self.make_branch('source')
-        Branch.hooks.install_hook('post_change_branch_tip',
-                                  self.capture_post_change_branch_tip_hook)
+        Branch.hooks.install_named_hook(
+            'post_change_branch_tip',
+            self.capture_post_change_branch_tip_hook,
+            None)
         branch.set_last_revision_info(0, NULL_REVISION)
         self.assertEqual(len(self.hook_calls), 1)
         self.assertEqual(self.hook_calls[0][0].branch, branch)
@@ -110,8 +117,10 @@
         tree.commit('empty commit', rev_id='foo')
         tree.unlock()
         branch = tree.branch
-        Branch.hooks.install_hook('post_change_branch_tip',
-                                  self.capture_post_change_branch_tip_hook)
+        Branch.hooks.install_named_hook(
+            'post_change_branch_tip',
+            self.capture_post_change_branch_tip_hook,
+            None)
         # some branches require that their history be set to a revision in the
         # repository
         branch.set_last_revision_info(1, 'f\xc2\xb5')
@@ -124,8 +133,10 @@
 
     def test_post_change_branch_tip_branch_is_locked(self):
         branch = self.make_branch('source')
-        Branch.hooks.install_hook('post_change_branch_tip',
-                                  self.capture_post_change_branch_tip_hook)
+        Branch.hooks.install_named_hook(
+            'post_change_branch_tip',
+            self.capture_post_change_branch_tip_hook,
+            None)
         branch.set_last_revision_info(0, NULL_REVISION)
         self.assertEqual(len(self.hook_calls), 1)
         self.assertEqual(self.hook_calls[0][0].branch, branch)
@@ -133,10 +144,14 @@
 
     def test_post_change_branch_tip_calls_all_hooks_no_errors(self):
         branch = self.make_branch('source')
-        Branch.hooks.install_hook('post_change_branch_tip',
-                                  self.capture_post_change_branch_tip_hook)
-        Branch.hooks.install_hook('post_change_branch_tip',
-                                  self.capture_post_change_branch_tip_hook)
+        Branch.hooks.install_named_hook(
+            'post_change_branch_tip',
+            self.capture_post_change_branch_tip_hook,
+            None)
+        Branch.hooks.install_named_hook(
+            'post_change_branch_tip',
+            self.capture_post_change_branch_tip_hook,
+            None)
         branch.set_last_revision_info(0, NULL_REVISION)
         self.assertEqual(len(self.hook_calls), 2)
         self.assertEqual(self.hook_calls[0][0].branch, branch)

=== modified file 'bzrlib/tests/branch_implementations/test_pull.py'
--- bzrlib/tests/branch_implementations/test_pull.py	2007-11-30 23:18:15 +0000
+++ bzrlib/tests/branch_implementations/test_pull.py	2008-03-10 13:37:31 +0000
@@ -130,7 +130,8 @@
     def test_post_pull_empty_history(self):
         target = self.make_branch('target')
         source = self.make_branch('source')
-        Branch.hooks.install_hook('post_pull', self.capture_post_pull_hook)
+        Branch.hooks.install_named_hook('post_pull',
+            self.capture_post_pull_hook, None)
         target.pull(source)
         # with nothing there we should still get a notification, and
         # have both branches locked at the notification time.
@@ -158,7 +159,8 @@
             local = BzrDir.create_branch_convenience('local2')
             local.bind(target)
         source = self.make_branch('source')
-        Branch.hooks.install_hook('post_pull', self.capture_post_pull_hook)
+        Branch.hooks.install_named_hook('post_pull',
+            self.capture_post_pull_hook, None)
         local.pull(source)
         # with nothing there we should still get a notification, and
         # have both branches locked at the notification time.
@@ -177,7 +179,8 @@
         sourcedir = target.bzrdir.clone(self.get_url('source'))
         source = MemoryTree.create_on_branch(sourcedir.open_branch())
         rev2 = source.commit('rev 2')
-        Branch.hooks.install_hook('post_pull', self.capture_post_pull_hook)
+        Branch.hooks.install_named_hook('post_pull',
+            self.capture_post_pull_hook, None)
         target.branch.pull(source.branch)
         # with nothing there we should still get a notification, and
         # have both branches locked at the notification time.

=== modified file 'bzrlib/tests/branch_implementations/test_push.py'
--- bzrlib/tests/branch_implementations/test_push.py	2007-11-01 09:52:45 +0000
+++ bzrlib/tests/branch_implementations/test_push.py	2008-03-10 13:30:29 +0000
@@ -182,7 +182,8 @@
     def test_post_push_empty_history(self):
         target = self.make_branch('target')
         source = self.make_branch('source')
-        Branch.hooks.install_hook('post_push', self.capture_post_push_hook)
+        Branch.hooks.install_named_hook('post_push',
+                                        self.capture_post_push_hook, None)
         source.push(target)
         # with nothing there we should still get a notification, and
         # have both branches locked at the notification time.
@@ -210,7 +211,8 @@
             local = BzrDir.create_branch_convenience('local2')
             local.bind(target)
         source = self.make_branch('source')
-        Branch.hooks.install_hook('post_push', self.capture_post_push_hook)
+        Branch.hooks.install_named_hook('post_push',
+                                        self.capture_post_push_hook, None)
         source.push(local)
         # with nothing there we should still get a notification, and
         # have both branches locked at the notification time.
@@ -229,7 +231,8 @@
         sourcedir = target.bzrdir.clone(self.get_url('source'))
         source = MemoryTree.create_on_branch(sourcedir.open_branch())
         rev2 = source.commit('rev 2')
-        Branch.hooks.install_hook('post_push', self.capture_post_push_hook)
+        Branch.hooks.install_named_hook('post_push',
+                                        self.capture_post_push_hook, None)
         source.branch.push(target.branch)
         # with nothing there we should still get a notification, and
         # have both branches locked at the notification time.

=== added file 'bzrlib/tests/branch_implementations/test_reconcile.py'
--- bzrlib/tests/branch_implementations/test_reconcile.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_reconcile.py	2008-04-29 16:43:14 +0000
@@ -0,0 +1,67 @@
+# Copyright (C) 2008 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for branch implementations - test reconcile() functionality"""
+
+from bzrlib import errors, reconcile
+from bzrlib.tests.branch_implementations import TestCaseWithBranch
+
+
+class TestBranchReconcile(TestCaseWithBranch):
+
+    def test_reconcile_fixes_invalid_revhistory(self):
+        # Different formats have different ways of handling invalid revision
+        # histories, so the setup portion is customized
+        tree = self.make_branch_and_tree('test')
+        r1 = tree.commit('one')
+        r2 = tree.commit('two')
+        r3 = tree.commit('three')
+        r4 = tree.commit('four')
+        # create an alternate branch
+        tree.set_parent_ids([r1])
+        tree.branch.set_last_revision_info(1, r1)
+        r2b = tree.commit('two-b')
+
+        # now go back and merge the commit
+        tree.set_parent_ids([r4, r2b])
+        tree.branch.set_last_revision_info(4, r4)
+
+        r5 = tree.commit('five')
+        # Now, try to set an invalid history
+        try:
+            tree.branch.set_revision_history([r1, r2b, r5])
+        except errors.NotLefthandHistory:
+            # Branch5 allows set_revision_history to be wrong
+            # Branch6 raises NotLefthandHistory, but we can force bogus stuff
+            # with set_last_revision_info
+            tree.branch.set_last_revision_info(3, r5)
+
+        self.assertEqual((3, r5), tree.branch.last_revision_info())
+        reconciler = tree.branch.reconcile()
+        self.assertEqual((5, r5), tree.branch.last_revision_info())
+        self.assertIs(True, reconciler.fixed_history)
+
+    def test_reconcile_returns_reconciler(self):
+        a_branch = self.make_branch('a_branch')
+        result = a_branch.reconcile()
+        self.assertIsInstance(result, reconcile.BranchReconciler)
+        # No history to fix
+        self.assertIs(False, result.fixed_history)
+
+    def test_reconcile_supports_thorough(self):
+        a_branch = self.make_branch('a_branch')
+        a_branch.reconcile(thorough=False)
+        a_branch.reconcile(thorough=True)

=== modified file 'bzrlib/tests/branch_implementations/test_uncommit.py'
--- bzrlib/tests/branch_implementations/test_uncommit.py	2007-07-11 14:58:54 +0000
+++ bzrlib/tests/branch_implementations/test_uncommit.py	2008-03-10 13:35:40 +0000
@@ -53,8 +53,8 @@
         tree.add('')
         revid = tree.commit('a revision')
         tree.unlock()
-        Branch.hooks.install_hook('post_uncommit',
-            self.capture_post_uncommit_hook)
+        Branch.hooks.install_named_hook('post_uncommit',
+            self.capture_post_uncommit_hook, None)
         uncommit(tree.branch)
         # with nothing left we should still get a notification, and
         # have the branch locked at notification time.
@@ -76,8 +76,8 @@
         tree.add('')
         revid = tree.commit('a revision')
         tree.unlock()
-        Branch.hooks.install_hook('post_uncommit',
-            self.capture_post_uncommit_hook)
+        Branch.hooks.install_named_hook('post_uncommit',
+            self.capture_post_uncommit_hook, None)
         uncommit(tree.branch)
         # with nothing left we should still get a notification, and
         # have the branch locked at notification time.
@@ -95,8 +95,8 @@
         revid2 = tree.commit('second revision')
         revid3 = tree.commit('third revision')
         tree.unlock()
-        Branch.hooks.install_hook('post_uncommit',
-            self.capture_post_uncommit_hook)
+        Branch.hooks.install_named_hook('post_uncommit',
+            self.capture_post_uncommit_hook, None)
         uncommit(tree.branch, revno=2)
         # having uncommitted from up the branch, we should get the
         # before and after revnos and revids correctly.

=== modified file 'bzrlib/tests/interrepository_implementations/__init__.py'
--- bzrlib/tests/interrepository_implementations/__init__.py	2008-03-24 15:44:44 +0000
+++ bzrlib/tests/interrepository_implementations/__init__.py	2008-04-25 21:36:59 +0000
@@ -25,6 +25,12 @@
 rather than in tests/interrepository_implementations/*.py.
 """
 
+
+from bzrlib.errors import (
+    FileExists,
+    UninitializableFormat,
+    )
+
 from bzrlib.repository import (
                                InterKnitRepo,
                                InterKnit1and2,
@@ -38,6 +44,8 @@
                           TestScenarioApplier,
                           TestSuite,
                           )
+from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.transport import get_transport
 
 
 class InterRepositoryTestProviderAdapter(TestScenarioApplier):
@@ -119,9 +127,46 @@
         return result
 
 
+class TestCaseWithInterRepository(TestCaseWithBzrDir):
+
+    def setUp(self):
+        super(TestCaseWithInterRepository, self).setUp()
+
+    def make_branch(self, relpath, format=None):
+        repo = self.make_repository(relpath, format=format)
+        return repo.bzrdir.create_branch()
+
+    def make_bzrdir(self, relpath, format=None):
+        try:
+            url = self.get_url(relpath)
+            segments = url.split('/')
+            if segments and segments[-1] not in ('', '.'):
+                parent = '/'.join(segments[:-1])
+                t = get_transport(parent)
+                try:
+                    t.mkdir(segments[-1])
+                except FileExists:
+                    pass
+            if format is None:
+                format = self.repository_format._matchingbzrdir
+            return format.initialize(url)
+        except UninitializableFormat:
+            raise TestSkipped("Format %s is not initializable." % format)
+
+    def make_repository(self, relpath, format=None):
+        made_control = self.make_bzrdir(relpath, format=format)
+        return self.repository_format.initialize(made_control)
+
+    def make_to_repository(self, relpath):
+        made_control = self.make_bzrdir(relpath,
+            self.repository_format_to._matchingbzrdir)
+        return self.repository_format_to.initialize(made_control)
+
+
 def test_suite():
     result = TestSuite()
     test_interrepository_implementations = [
+        'bzrlib.tests.interrepository_implementations.test_fetch',
         'bzrlib.tests.interrepository_implementations.test_interrepository',
         ]
     adapter = InterRepositoryTestProviderAdapter(

=== added file 'bzrlib/tests/interrepository_implementations/test_fetch.py'
--- bzrlib/tests/interrepository_implementations/test_fetch.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/interrepository_implementations/test_fetch.py	2008-04-27 23:47:45 +0000
@@ -0,0 +1,216 @@
+# Copyright (C) 2005, 2006, 2008 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import sys
+
+import bzrlib
+from bzrlib import (
+    errors,
+    repository,
+    osutils,
+    )
+from bzrlib.errors import (
+    NoSuchRevision,
+    )
+from bzrlib.revision import (
+    NULL_REVISION,
+    Revision,
+    )
+from bzrlib.tests import (
+    TestNotApplicable,
+    )
+from bzrlib.tests.interrepository_implementations import (
+    TestCaseWithInterRepository,
+    )
+
+
+class TestInterRepository(TestCaseWithInterRepository):
+
+    def test_fetch(self):
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/foo'])
+        tree_a.add('foo', 'file1')
+        tree_a.commit('rev1', rev_id='rev1')
+        def check_push_rev1(repo):
+            # ensure the revision is missing.
+            self.assertRaises(NoSuchRevision, repo.get_revision, 'rev1')
+            # fetch with a limit of NULL_REVISION and an explicit progress bar.
+            repo.fetch(tree_a.branch.repository,
+                       revision_id=NULL_REVISION,
+                       pb=bzrlib.progress.DummyProgress())
+            # nothing should have been pushed
+            self.assertFalse(repo.has_revision('rev1'))
+            # fetch with a default limit (grab everything)
+            repo.fetch(tree_a.branch.repository)
+            # check that b now has all the data from a's first commit.
+            rev = repo.get_revision('rev1')
+            tree = repo.revision_tree('rev1')
+            tree.lock_read()
+            self.addCleanup(tree.unlock)
+            tree.get_file_text('file1')
+            for file_id in tree:
+                if tree.inventory[file_id].kind == "file":
+                    tree.get_file(file_id).read()
+
+        # makes a target version repo 
+        repo_b = self.make_to_repository('b')
+        check_push_rev1(repo_b)
+
+    def test_fetch_missing_basis_text(self):
+        """If fetching a delta, we should die if a basis is not present."""
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add(['a'], ['a-id'])
+        tree.commit('one', rev_id='rev-one')
+        self.build_tree_contents([('tree/a', 'new contents\n')])
+        tree.commit('two', rev_id='rev-two')
+
+        to_repo = self.make_to_repository('to_repo')
+        # We build a broken revision so that we can test the fetch code dies
+        # properly. So copy the inventory and revision, but not the text.
+        to_repo.lock_write()
+        try:
+            to_repo.start_write_group()
+            inv = tree.branch.repository.get_inventory('rev-one')
+            to_repo.add_inventory('rev-one', inv, [])
+            rev = tree.branch.repository.get_revision('rev-one')
+            to_repo.add_revision('rev-one', rev, inv=inv)
+            to_repo.commit_write_group()
+        finally:
+            to_repo.unlock()
+
+        # Implementations can either copy the missing basis text, or raise an
+        # exception
+        try:
+            to_repo.fetch(tree.branch.repository, 'rev-two')
+        except errors.RevisionNotPresent, e:
+            # If an exception is raised, the revision should not be in the
+            # target.
+            self.assertRaises(
+                (errors.NoSuchRevision, errors.RevisionNotPresent),
+                 to_repo.revision_tree, 'rev-two')
+        else:
+            # If not exception is raised, then the basis text should be
+            # available.
+            to_repo.lock_read()
+            try:
+                rt = to_repo.revision_tree('rev-one')
+                self.assertEqual('contents of tree/a\n',
+                                 rt.get_file_text('a-id'))
+            finally:
+                to_repo.unlock()
+
+    def test_fetch_missing_revision_same_location_fails(self):
+        repo_a = self.make_repository('.')
+        repo_b = repository.Repository.open('.')
+        try:
+            self.assertRaises(errors.NoSuchRevision, repo_b.fetch, repo_a,
+                              revision_id='XXX')
+        except errors.LockError, e:
+            check_old_format_lock_error(self.repository_format)
+
+    def test_fetch_same_location_trivial_works(self):
+        repo_a = self.make_repository('.')
+        repo_b = repository.Repository.open('.')
+        try:
+            repo_a.fetch(repo_b)
+        except errors.LockError, e:
+            check_old_format_lock_error(self.repository_format)
+
+    def test_fetch_missing_text_other_location_fails(self):
+        source_tree = self.make_branch_and_tree('source')
+        source = source_tree.branch.repository
+        target = self.make_to_repository('target')
+    
+        # start by adding a file so the data knit for the file exists in
+        # repositories that have specific files for each fileid.
+        self.build_tree(['source/id'])
+        source_tree.add(['id'], ['id'])
+        source_tree.commit('a', rev_id='a')
+        # now we manually insert a revision with an inventory referencing
+        # 'id' at revision 'b', but we do not insert revision b.
+        # this should ensure that the new versions of files are being checked
+        # for during pull operations
+        inv = source.get_inventory('a')
+        source.lock_write()
+        self.addCleanup(source.unlock)
+        source.start_write_group()
+        inv['id'].revision = 'b'
+        inv.revision_id = 'b'
+        sha1 = source.add_inventory('b', inv, ['a'])
+        rev = Revision(timestamp=0,
+                       timezone=None,
+                       committer="Foo Bar <foo@example.com>",
+                       message="Message",
+                       inventory_sha1=sha1,
+                       revision_id='b')
+        rev.parent_ids = ['a']
+        source.add_revision('b', rev)
+        source.commit_write_group()
+        self.assertRaises(errors.RevisionNotPresent, target.fetch, source)
+        self.assertFalse(target.has_revision('b'))
+
+    def test_fetch_funky_file_id(self):
+        from_tree = self.make_branch_and_tree('tree')
+        if sys.platform == 'win32':
+            from_repo = from_tree.branch.repository
+            check_repo_format_for_funky_id_on_win32(from_repo)
+        self.build_tree(['tree/filename'])
+        from_tree.add('filename', 'funky-chars<>%&;"\'')
+        from_tree.commit('commit filename')
+        to_repo = self.make_to_repository('to')
+        to_repo.fetch(from_tree.branch.repository,
+                      from_tree.get_parent_ids()[0])
+
+    def test_fetch_revision_hash(self):
+        """Ensure that inventory hashes are updated by fetch"""
+        from_tree = self.make_branch_and_tree('tree')
+        from_tree.commit('foo', rev_id='foo-id')
+        to_repo = self.make_to_repository('to')
+        to_repo.fetch(from_tree.branch.repository)
+        recorded_inv_sha1 = to_repo.get_inventory_sha1('foo-id')
+        xml = to_repo.get_inventory_xml('foo-id')
+        computed_inv_sha1 = osutils.sha_string(xml)
+        self.assertEqual(computed_inv_sha1, recorded_inv_sha1)
+
+
+class TestFetchDependentData(TestCaseWithInterRepository):
+
+    def test_reference(self):
+        from_tree = self.make_branch_and_tree('tree')
+        to_repo = self.make_to_repository('to')
+        if (not from_tree.supports_tree_reference() or
+            not from_tree.branch.repository._format.supports_tree_reference or
+            not to_repo._format.supports_tree_reference):
+            raise TestNotApplicable("Need subtree support.")
+        subtree = self.make_branch_and_tree('tree/subtree')
+        subtree.commit('subrev 1')
+        from_tree.add_reference(subtree)
+        tree_rev = from_tree.commit('foo')
+        # now from_tree has a last-modified of subtree of the rev id of the
+        # commit for foo, and a reference revision of the rev id of the commit
+        # for subrev 1
+        to_repo.fetch(from_tree.branch.repository, tree_rev)
+        # to_repo should have a file_graph for from_tree.path2id('subtree') and
+        # revid tree_rev.
+        to_repo.lock_read()
+        try:
+            file_vf = to_repo.weave_store.get_weave(
+                from_tree.path2id('subtree'), to_repo.get_transaction())
+            self.assertEqual([tree_rev], file_vf.get_ancestry([tree_rev]))
+        finally:
+            to_repo.unlock()

=== modified file 'bzrlib/tests/interrepository_implementations/test_interrepository.py'
--- bzrlib/tests/interrepository_implementations/test_interrepository.py	2008-04-26 06:07:28 +0000
+++ bzrlib/tests/interrepository_implementations/test_interrepository.py	2008-04-27 22:35:42 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,12 +22,6 @@
 import bzrlib.bzrdir as bzrdir
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
 import bzrlib.errors as errors
-from bzrlib.errors import (FileExists,
-                           NoSuchRevision,
-                           NoSuchFile,
-                           UninitializableFormat,
-                           NotBranchError,
-                           )
 import bzrlib.gpg
 from bzrlib.inventory import Inventory
 import bzrlib.repofmt.weaverepo as weaverepo
@@ -40,44 +34,9 @@
     TestNotApplicable,
     TestSkipped,
     )
-from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
-from bzrlib.transport import get_transport
-
-
-class TestCaseWithInterRepository(TestCaseWithBzrDir):
-
-    def setUp(self):
-        super(TestCaseWithInterRepository, self).setUp()
-
-    def make_branch(self, relpath, format=None):
-        repo = self.make_repository(relpath, format=format)
-        return repo.bzrdir.create_branch()
-
-    def make_bzrdir(self, relpath, format=None):
-        try:
-            url = self.get_url(relpath)
-            segments = url.split('/')
-            if segments and segments[-1] not in ('', '.'):
-                parent = '/'.join(segments[:-1])
-                t = get_transport(parent)
-                try:
-                    t.mkdir(segments[-1])
-                except FileExists:
-                    pass
-            if format is None:
-                format = self.repository_format._matchingbzrdir
-            return format.initialize(url)
-        except UninitializableFormat:
-            raise TestSkipped("Format %s is not initializable." % format)
-
-    def make_repository(self, relpath, format=None):
-        made_control = self.make_bzrdir(relpath, format=format)
-        return self.repository_format.initialize(made_control)
-
-    def make_to_repository(self, relpath):
-        made_control = self.make_bzrdir(relpath,
-            self.repository_format_to._matchingbzrdir)
-        return self.repository_format_to.initialize(made_control)
+from bzrlib.tests.interrepository_implementations import (
+    TestCaseWithInterRepository,
+    )
 
 
 def check_old_format_lock_error(repository_format):
@@ -131,138 +90,6 @@
         ## self.assertEqual(self.interrepo_class, interrepo.__class__)
         pass
 
-    def test_fetch(self):
-        tree_a = self.make_branch_and_tree('a')
-        self.build_tree(['a/foo'])
-        tree_a.add('foo', 'file1')
-        tree_a.commit('rev1', rev_id='rev1')
-        def check_push_rev1(repo):
-            # ensure the revision is missing.
-            self.assertRaises(NoSuchRevision, repo.get_revision, 'rev1')
-            # fetch with a limit of NULL_REVISION and an explicit progress bar.
-            repo.fetch(tree_a.branch.repository,
-                       revision_id=NULL_REVISION,
-                       pb=bzrlib.progress.DummyProgress())
-            # nothing should have been pushed
-            self.assertFalse(repo.has_revision('rev1'))
-            # fetch with a default limit (grab everything)
-            repo.fetch(tree_a.branch.repository)
-            # check that b now has all the data from a's first commit.
-            rev = repo.get_revision('rev1')
-            tree = repo.revision_tree('rev1')
-            tree.lock_read()
-            self.addCleanup(tree.unlock)
-            tree.get_file_text('file1')
-            for file_id in tree:
-                if tree.inventory[file_id].kind == "file":
-                    tree.get_file(file_id).read()
-
-        # makes a target version repo 
-        repo_b = self.make_to_repository('b')
-        check_push_rev1(repo_b)
-
-    def test_fetch_missing_basis_text(self):
-        """If fetching a delta, we should die if a basis is not present."""
-        tree = self.make_branch_and_tree('tree')
-        self.build_tree(['tree/a'])
-        tree.add(['a'], ['a-id'])
-        tree.commit('one', rev_id='rev-one')
-        self.build_tree_contents([('tree/a', 'new contents\n')])
-        tree.commit('two', rev_id='rev-two')
-
-        to_repo = self.make_to_repository('to_repo')
-        # We build a broken revision so that we can test the fetch code dies
-        # properly. So copy the inventory and revision, but not the text.
-        to_repo.lock_write()
-        try:
-            to_repo.start_write_group()
-            inv = tree.branch.repository.get_inventory('rev-one')
-            to_repo.add_inventory('rev-one', inv, [])
-            rev = tree.branch.repository.get_revision('rev-one')
-            to_repo.add_revision('rev-one', rev, inv=inv)
-            to_repo.commit_write_group()
-        finally:
-            to_repo.unlock()
-
-        # Implementations can either copy the missing basis text, or raise an
-        # exception
-        try:
-            to_repo.fetch(tree.branch.repository, 'rev-two')
-        except errors.RevisionNotPresent, e:
-            # If an exception is raised, the revision should not be in the
-            # target.
-            self.assertRaises((errors.NoSuchRevision, errors.RevisionNotPresent),
-                              to_repo.revision_tree, 'rev-two')
-        else:
-            # If not exception is raised, then the basis text should be
-            # available.
-            to_repo.lock_read()
-            try:
-                rt = to_repo.revision_tree('rev-one')
-                self.assertEqual('contents of tree/a\n',
-                                 rt.get_file_text('a-id'))
-            finally:
-                to_repo.unlock()
-
-    def test_fetch_missing_revision_same_location_fails(self):
-        repo_a = self.make_repository('.')
-        repo_b = repository.Repository.open('.')
-        try:
-            self.assertRaises(errors.NoSuchRevision, repo_b.fetch, repo_a, revision_id='XXX')
-        except errors.LockError, e:
-            check_old_format_lock_error(self.repository_format)
-
-    def test_fetch_same_location_trivial_works(self):
-        repo_a = self.make_repository('.')
-        repo_b = repository.Repository.open('.')
-        try:
-            repo_a.fetch(repo_b)
-        except errors.LockError, e:
-            check_old_format_lock_error(self.repository_format)
-
-    def test_fetch_missing_text_other_location_fails(self):
-        source_tree = self.make_branch_and_tree('source')
-        source = source_tree.branch.repository
-        target = self.make_to_repository('target')
-    
-        # start by adding a file so the data knit for the file exists in
-        # repositories that have specific files for each fileid.
-        self.build_tree(['source/id'])
-        source_tree.add(['id'], ['id'])
-        source_tree.commit('a', rev_id='a')
-        # now we manually insert a revision with an inventory referencing
-        # 'id' at revision 'b', but we do not insert revision b.
-        # this should ensure that the new versions of files are being checked
-        # for during pull operations
-        inv = source.get_inventory('a')
-        source.lock_write()
-        self.addCleanup(source.unlock)
-        source.start_write_group()
-        inv['id'].revision = 'b'
-        inv.revision_id = 'b'
-        sha1 = source.add_inventory('b', inv, ['a'])
-        rev = Revision(timestamp=0,
-                       timezone=None,
-                       committer="Foo Bar <foo@example.com>",
-                       message="Message",
-                       inventory_sha1=sha1,
-                       revision_id='b')
-        rev.parent_ids = ['a']
-        source.add_revision('b', rev)
-        source.commit_write_group()
-        self.assertRaises(errors.RevisionNotPresent, target.fetch, source)
-        self.assertFalse(target.has_revision('b'))
-
-    def test_fetch_funky_file_id(self):
-        from_tree = self.make_branch_and_tree('tree')
-        if sys.platform == 'win32':
-            from_repo = from_tree.branch.repository
-            check_repo_format_for_funky_id_on_win32(from_repo)
-        self.build_tree(['tree/filename'])
-        from_tree.add('filename', 'funky-chars<>%&;"\'')
-        from_tree.commit('commit filename')
-        to_repo = self.make_to_repository('to')
-        to_repo.fetch(from_tree.branch.repository, from_tree.get_parent_ids()[0])
 
 
 class TestCaseWithComplexRepository(TestCaseWithInterRepository):
@@ -325,10 +152,12 @@
         self.assertFalse(repo_a.has_revision('pizza'))
         self.assertFalse(repo_b.has_revision('pizza'))
         # Asking specifically for an absent revision errors.
-        self.assertRaises(NoSuchRevision, repo_b.search_missing_revision_ids, repo_a,
-            revision_id='pizza', find_ghosts=True)
-        self.assertRaises(NoSuchRevision, repo_b.search_missing_revision_ids, repo_a,
-            revision_id='pizza', find_ghosts=False)
+        self.assertRaises(errors.NoSuchRevision,
+            repo_b.search_missing_revision_ids, repo_a, revision_id='pizza',
+            find_ghosts=True)
+        self.assertRaises(errors.NoSuchRevision,
+            repo_b.search_missing_revision_ids, repo_a, revision_id='pizza',
+            find_ghosts=False)
 
     def test_search_missing_revision_ids_revision_limited(self):
         # revision ids in repository A that are not referenced by the
@@ -404,31 +233,3 @@
         # rev must not be corrupt now
         self.assertEqual([None, 'ghost', 'references', 'tip'],
             missing_ghost.get_ancestry('tip'))
-
-
-class TestFetchDependentData(TestCaseWithInterRepository):
-
-    def test_reference(self):
-        from_tree = self.make_branch_and_tree('tree')
-        to_repo = self.make_to_repository('to')
-        if (not from_tree.supports_tree_reference() or
-            not from_tree.branch.repository._format.supports_tree_reference or
-            not to_repo._format.supports_tree_reference):
-            raise TestNotApplicable("Need subtree support.")
-        subtree = self.make_branch_and_tree('tree/subtree')
-        subtree.commit('subrev 1')
-        from_tree.add_reference(subtree)
-        tree_rev = from_tree.commit('foo')
-        # now from_tree has a last-modified of subtree of the rev id of the
-        # commit for foo, and a reference revision of the rev id of the commit
-        # for subrev 1
-        to_repo.fetch(from_tree.branch.repository, tree_rev)
-        # to_repo should have a file_graph for from_tree.path2id('subtree') and
-        # revid tree_rev.
-        to_repo.lock_read()
-        try:
-            file_vf = to_repo.weave_store.get_weave(
-                from_tree.path2id('subtree'), to_repo.get_transaction())
-            self.assertEqual([tree_rev], file_vf.get_ancestry([tree_rev]))
-        finally:
-            to_repo.unlock()

=== modified file 'bzrlib/tests/repository_implementations/test_fetch.py'
--- bzrlib/tests/repository_implementations/test_fetch.py	2008-04-10 06:08:55 +0000
+++ bzrlib/tests/repository_implementations/test_fetch.py	2008-04-25 22:12:34 +0000
@@ -141,3 +141,11 @@
         # without causing any errors.
         target_repo.fetch(source_repo, revision_id='rev1')
 
+    def test_fetch_all_same_revisions_twice(self):
+        # Blind-fetching all the same revisions twice should succeed and be a
+        # no-op the second time.
+        repo = self.make_repository('repo')
+        tree = self.make_branch_and_tree('tree')
+        revision_id = tree.commit('test')
+        repo.fetch(tree.branch.repository)
+        repo.fetch(tree.branch.repository)

=== modified file 'bzrlib/tests/test_bundle.py'
--- bzrlib/tests/test_bundle.py	2008-03-26 03:06:52 +0000
+++ bzrlib/tests/test_bundle.py	2008-04-28 03:55:56 +0000
@@ -42,7 +42,7 @@
                            NoSuchFile,)
 from bzrlib.merge import Merge3Merger
 from bzrlib.repofmt import knitrepo
-from bzrlib.osutils import sha_file
+from bzrlib.osutils import sha_file, sha_string
 from bzrlib.tests import (
     SymlinkFeature,
     TestCase,
@@ -1015,19 +1015,29 @@
         self.assertNotContainsRe(inv_text, 'format="5"')
         self.assertContainsRe(inv_text, 'format="7"')
 
+    def make_repo_with_installed_revisions(self):
+        tree = self.make_simple_tree('knit')
+        tree.commit('hello', rev_id='rev1')
+        tree.commit('hello', rev_id='rev2')
+        bundle = read_bundle(self.create_bundle_text('null:', 'rev2')[0])
+        repo = self.make_repository('repo', format='dirstate-with-subtree')
+        bundle.install_revisions(repo)
+        return repo
+
     def test_across_models(self):
-        tree = self.make_simple_tree('knit')
-        tree.commit('hello', rev_id='rev1')
-        tree.commit('hello', rev_id='rev2')
-        bundle = read_bundle(self.create_bundle_text('null:', 'rev2')[0])
-        repo = self.make_repository('repo', format='dirstate-with-subtree')
-        bundle.install_revisions(repo)
+        repo = self.make_repo_with_installed_revisions()
         inv = repo.get_inventory('rev2')
         self.assertEqual('rev2', inv.root.revision)
         root_vf = repo.weave_store.get_weave(inv.root.file_id,
                                              repo.get_transaction())
         self.assertEqual(root_vf.versions(), ['rev1', 'rev2'])
 
+    def test_inv_hash_across_serializers(self):
+        repo = self.make_repo_with_installed_revisions()
+        recorded_inv_sha1 = repo.get_inventory_sha1('rev2')
+        xml = repo.get_inventory_xml('rev2')
+        self.assertEqual(sha_string(xml), recorded_inv_sha1)
+
     def test_across_models_incompatible(self):
         tree = self.make_simple_tree('dirstate-with-subtree')
         tree.commit('hello', rev_id='rev1')

=== modified file 'bzrlib/tests/test_fetch.py'
--- bzrlib/tests/test_fetch.py	2008-04-08 00:57:07 +0000
+++ bzrlib/tests/test_fetch.py	2008-04-30 04:50:30 +0000
@@ -322,3 +322,75 @@
         self.assertTrue(1 >= self._count_log_matches('last-revision',
                                                      http_logs))
         self.assertEqual(4, len(http_logs))
+
+
+class Test1To2Fetch(TestCaseWithTransport):
+    """Tests for Model1To2 failure modes"""
+
+    def make_tree_and_repo(self):
+        self.tree = self.make_branch_and_tree('tree', format='pack-0.92')
+        self.repo = self.make_repository('rich-repo', format='rich-root-pack')
+        self.repo.lock_write()
+        self.addCleanup(self.repo.unlock)
+
+    def do_fetch_order_test(self, first, second):
+        """Test that fetch works no matter what the set order of revision is.
+
+        This test depends on the order of items in a set, which is
+        implementation-dependant, so we test A, B and then B, A.
+        """
+        self.make_tree_and_repo()
+        self.tree.commit('Commit 1', rev_id=first)
+        self.tree.commit('Commit 2', rev_id=second)
+        self.repo.fetch(self.tree.branch.repository, second)
+
+    def test_fetch_order_AB(self):
+        """See do_fetch_order_test"""
+        self.do_fetch_order_test('A', 'B')
+
+    def test_fetch_order_BA(self):
+        """See do_fetch_order_test"""
+        self.do_fetch_order_test('B', 'A')
+
+    def get_parents(self, file_id, revision_id):
+        transaction = self.repo.get_transaction()
+        vf = self.repo.weave_store.get_weave(file_id, transaction)
+        return vf.get_parents_with_ghosts(revision_id)
+
+    def test_fetch_ghosts(self):
+        self.make_tree_and_repo()
+        self.tree.commit('first commit', rev_id='left-parent')
+        self.tree.add_parent_tree_id('ghost-parent')
+        fork = self.tree.bzrdir.sprout('fork', 'null:').open_workingtree()
+        fork.commit('not a ghost', rev_id='not-ghost-parent')
+        self.tree.branch.repository.fetch(fork.branch.repository,
+                                     'not-ghost-parent')
+        self.tree.add_parent_tree_id('not-ghost-parent')
+        self.tree.commit('second commit', rev_id='second-id')
+        self.repo.fetch(self.tree.branch.repository, 'second-id')
+        root_id = self.tree.get_root_id()
+        self.assertEqual(['left-parent', 'ghost-parent', 'not-ghost-parent'],
+                         self.get_parents(root_id, 'second-id'))
+
+    def make_two_commits(self, change_root, fetch_twice):
+        self.make_tree_and_repo()
+        self.tree.commit('first commit', rev_id='first-id')
+        if change_root:
+            self.tree.set_root_id('unique-id')
+        self.tree.commit('second commit', rev_id='second-id')
+        if fetch_twice:
+            self.repo.fetch(self.tree.branch.repository, 'first-id')
+        self.repo.fetch(self.tree.branch.repository, 'second-id')
+
+    def test_fetch_changed_root(self):
+        self.make_two_commits(change_root=True, fetch_twice=False)
+        self.assertEqual([], self.get_parents('unique-id', 'second-id'))
+
+    def test_two_fetch_changed_root(self):
+        self.make_two_commits(change_root=True, fetch_twice=True)
+        self.assertEqual([], self.get_parents('unique-id', 'second-id'))
+
+    def test_two_fetches(self):
+        self.make_two_commits(change_root=False, fetch_twice=True)
+        self.assertEqual(['first-id'],
+                         self.get_parents('TREE_ROOT', 'second-id'))

=== modified file 'bzrlib/tests/test_hooks.py'
--- bzrlib/tests/test_hooks.py	2007-06-28 03:08:53 +0000
+++ bzrlib/tests/test_hooks.py	2008-04-30 22:56:47 +0000
@@ -23,6 +23,7 @@
     UnknownHook,
     )
 
+from bzrlib.symbol_versioning import one_five
 from bzrlib.tests import TestCase
 
 
@@ -31,25 +32,43 @@
     def test_install_hook_raises_unknown_hook(self):
         """install_hook should raise UnknownHook if a hook is unknown."""
         hooks = Hooks()
-        self.assertRaises(UnknownHook, hooks.install_hook, 'silly', None)
+        self.assertRaises(UnknownHook, self.applyDeprecated, one_five,
+                          hooks.install_hook, 'silly', None)
 
     def test_install_hook_appends_known_hook(self):
         """install_hook should append the callable for known hooks."""
         hooks = Hooks()
         hooks['set_rh'] = []
-        hooks.install_hook('set_rh', None)
-        self.assertEqual(hooks['set_rh'], [None])
+        self.applyDeprecated(one_five, hooks.install_hook, 'set_rh', None)
+        self.assertEqual(hooks['set_rh'], [None])
+
+    def test_install_named_hook_raises_unknown_hook(self):
+        hooks = Hooks()
+        self.assertRaises(UnknownHook, hooks.install_named_hook, 'silly',
+                          None, "")
+
+    def test_install_named_hook_appends_known_hook(self):
+        hooks = Hooks()
+        hooks['set_rh'] = []
+        hooks.install_named_hook('set_rh', None, "demo")
+        self.assertEqual(hooks['set_rh'], [None])
+
+    def test_install_named_hook_and_retrieve_name(self):
+        hooks = Hooks()
+        hooks['set_rh'] = []
+        hooks.install_named_hook('set_rh', None, "demo")
+        self.assertEqual("demo", hooks.get_hook_name(None))
 
     def test_name_hook_and_retrieve_name(self):
         """name_hook puts the name in the names mapping."""
         hooks = Hooks()
         hooks['set_rh'] = []
-        hooks.install_hook('set_rh', None)
+        self.applyDeprecated(one_five, hooks.install_hook, 'set_rh', None)
         hooks.name_hook(None, 'demo')
         self.assertEqual("demo", hooks.get_hook_name(None))
 
     def test_get_unnamed_hook_name_is_unnamed(self):
         hooks = Hooks()
         hooks['set_rh'] = []
-        hooks.install_hook('set_rh', None)
+        self.applyDeprecated(one_five, hooks.install_hook, 'set_rh', None)
         self.assertEqual("No hook name", hooks.get_hook_name(None))

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2008-04-09 23:35:55 +0000
+++ bzrlib/tests/test_knit.py	2008-04-28 06:24:16 +0000
@@ -1505,6 +1505,10 @@
         for plan_line, expected_line in zip(plan, AB_MERGE):
             self.assertEqual(plan_line, expected_line)
 
+
+class GetDataStreamTests(KnitTests):
+    """Tests for get_data_stream."""
+
     def test_get_stream_empty(self):
         """Get a data stream for an empty knit file."""
         k1 = self.make_test_knit()
@@ -1696,6 +1700,10 @@
             bytes = reader_callable(length)
             self.assertRecordContentEqual(k1, version_id, bytes)
 
+
+class InsertDataStreamTests(KnitTests):
+    """Tests for insert_data_stream."""
+
     def assertKnitFilesEqual(self, knit1, knit2):
         """Assert that the contents of the index and data files of two knits are
         equal.
@@ -1720,7 +1728,7 @@
             self.assertEqual(left.annotate(version),
                 right.annotate(version))
 
-    def test_insert_data_stream_empty(self):
+    def test_empty_stream(self):
         """Inserting a data stream with no records should not put any data into
         the knit.
         """
@@ -1733,7 +1741,7 @@
                          k1.transport.get_bytes(k1._index._filename),
                          "The .kndx should have nothing apart from the header.")
 
-    def test_insert_data_stream_one_record(self):
+    def test_one_record(self):
         """Inserting a data stream with one record from a knit with one record
         results in byte-identical files.
         """
@@ -1744,7 +1752,7 @@
         target.insert_data_stream(data_stream)
         self.assertKnitFilesEqual(source, target)
 
-    def test_insert_data_stream_annotated_unannotated(self):
+    def test_annotated_stream_into_unannotated_knit(self):
         """Inserting an annotated datastream to an unannotated knit works."""
         # case one - full texts.
         source = self.make_test_knit(name='source', annotate=True)
@@ -1757,7 +1765,7 @@
         target.insert_data_stream(source.get_data_stream(['text-b']))
         self.assertKnitValuesEqual(source, target)
 
-    def test_insert_data_stream_unannotated_annotated(self):
+    def test_unannotated_stream_into_annotated_knit(self):
         """Inserting an unannotated datastream to an annotated knit works."""
         # case one - full texts.
         source = self.make_test_knit(name='source', annotate=False)
@@ -1770,7 +1778,7 @@
         target.insert_data_stream(source.get_data_stream(['text-b']))
         self.assertKnitValuesEqual(source, target)
 
-    def test_insert_data_stream_records_already_present(self):
+    def test_records_already_present(self):
         """Insert a data stream where some records are alreday present in the
         target, and some not.  Only the new records are inserted.
         """
@@ -1788,7 +1796,7 @@
         # record was not added a second time.
         self.assertKnitFilesEqual(source, target)
 
-    def test_insert_data_stream_multiple_records(self):
+    def test_multiple_records(self):
         """Inserting a data stream of all records from a knit with multiple
         records results in byte-identical files.
         """
@@ -1803,7 +1811,7 @@
         
         self.assertKnitFilesEqual(source, target)
 
-    def test_insert_data_stream_ghost_parent(self):
+    def test_ghost_parent(self):
         """Insert a data stream with a record that has a ghost parent."""
         # Make a knit with a record, text-a, that has a ghost parent.
         source = self.make_test_knit(name='source')
@@ -1824,7 +1832,7 @@
             target.get_parent_map(['text-a', 'text-ghost']))
         self.assertEqual(split_lines(TEXT_1), target.get_lines('text-a'))
 
-    def test_insert_data_stream_inconsistent_version_lines(self):
+    def test_inconsistent_version_lines(self):
         """Inserting a data stream which has different content for a version_id
         than already exists in the knit will raise KnitCorrupt.
         """
@@ -1838,7 +1846,7 @@
         self.assertRaises(
             errors.KnitCorrupt, target.insert_data_stream, data_stream)
 
-    def test_insert_data_stream_inconsistent_version_parents(self):
+    def test_inconsistent_version_parents(self):
         """Inserting a data stream which has different parents for a version_id
         than already exists in the knit will raise KnitCorrupt.
         """
@@ -1853,7 +1861,7 @@
         self.assertRaises(
             errors.KnitCorrupt, target.insert_data_stream, data_stream)
 
-    def test_insert_data_stream_unknown_format(self):
+    def test_unknown_stream_format(self):
         """A data stream in a different format to the target knit cannot be
         inserted.
 
@@ -1867,7 +1875,7 @@
             errors.KnitDataStreamUnknown,
             target.insert_data_stream, data_stream)
 
-    def test_insert_data_stream_bug_208418(self):
+    def test_bug_208418(self):
         """You can insert a stream with an incompatible format, even when:
           * the stream has a line-delta record,
           * whose parent is in the target, also stored as a line-delta
@@ -1897,10 +1905,26 @@
         target.insert_data_stream(data_stream)
         # No errors should have been raised.
 
-
-    #  * test that a stream of "already present version, then new version"
-    #    inserts correctly.
-
+    def test_line_delta_record_into_non_delta_knit(self):
+        # Make a data stream with a line-delta record
+        source = self.make_test_knit(name='source', delta=True)
+        base_lines = split_lines(TEXT_1)
+        source.add_lines('version-1', [], base_lines)
+        source.add_lines('version-2', ['version-1'], base_lines + ['a\n'])
+        # The second record should be a delta.
+        self.assertEqual('line-delta', source._index.get_method('version-2'))
+        data_stream = source.get_data_stream(['version-1', 'version-2'])
+
+        # Insert the stream into a non-delta knit.
+        target = self.make_test_knit(name='target', delta=False)
+        target.insert_data_stream(data_stream)
+        
+        # Both versions are fulltexts in the target
+        self.assertEqual('fulltext', target._index.get_method('version-1'))
+        self.assertEqual('fulltext', target._index.get_method('version-2'))
+
+
+class DataStreamTests(KnitTests):
 
     def assertMadeStreamKnit(self, source_knit, versions, target_knit):
         """Assert that a knit made from a stream is as expected."""

=== modified file 'bzrlib/tests/test_reconcile.py'
--- bzrlib/tests/test_reconcile.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/test_reconcile.py	2008-04-30 15:02:49 +0000
@@ -17,8 +17,7 @@
 """Tests for reconiliation behaviour that is repository independent."""
 
 
-import bzrlib
-import bzrlib.errors as errors
+from bzrlib import bzrdir, errors, tests
 from bzrlib.reconcile import reconcile, Reconciler
 from bzrlib.revision import Revision
 from bzrlib.tests.repository_implementations.test_repository import TestCaseWithRepository
@@ -30,10 +29,10 @@
 
     def test_reweave_empty(self):
         # we want a repo capable format
-        parent = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('.')
+        parent = bzrdir.BzrDirMetaFormat1().initialize('.')
         parent.create_repository(shared=True)
         parent.root_transport.mkdir('child')
-        child = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('child')
+        child = bzrdir.BzrDirMetaFormat1().initialize('child')
         self.assertRaises(errors.NoRepositoryPresent, child.open_repository)
         reconciler = Reconciler(child)
         reconciler.reconcile()
@@ -44,3 +43,27 @@
         self.assertEqual(0, reconciler.inconsistent_parents)
         # and no garbage inventories
         self.assertEqual(0, reconciler.garbage_inventories)
+
+
+class TestReconciler(tests.TestCaseWithTransport):
+    
+    def test_reconciler_with_no_branch(self):
+        repo = self.make_repository('repo')
+        reconciler = Reconciler(repo.bzrdir)
+        reconciler.reconcile()
+        # no inconsistent parents should have been found
+        # but the values should have been set.
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        # and no garbage inventories
+        self.assertEqual(0, reconciler.garbage_inventories)
+        self.assertIs(None, reconciler.fixed_branch_history)
+
+    def test_reconciler_finds_branch(self):
+        a_branch = self.make_branch('a_branch')
+        reconciler = Reconciler(a_branch.bzrdir)
+        reconciler.reconcile()
+
+        # It should have checked the repository, and the branch
+        self.assertEqual(0, reconciler.inconsistent_parents)
+        self.assertEqual(0, reconciler.garbage_inventories)
+        self.assertIs(False, reconciler.fixed_branch_history)

=== modified file 'bzrlib/tests/test_reconfigure.py'
--- bzrlib/tests/test_reconfigure.py	2008-04-24 04:58:42 +0000
+++ bzrlib/tests/test_reconfigure.py	2008-04-25 22:16:00 +0000
@@ -201,10 +201,6 @@
         parent = self.make_branch('parent')
         checkout = parent.create_checkout('checkout')
         checkout.commit('test', rev_id='new-commit', local=True)
-
-        # work around fetch bug 212908
-        checkout.commit('test2', local=True)
-        checkout.branch.set_last_revision_info(1, 'new-commit')
         reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
             checkout.bzrdir)
         return checkout, parent, reconfiguration
@@ -229,9 +225,6 @@
         child = parent.bzrdir.sprout('child').open_workingtree()
         child.commit('test', rev_id='new-commit')
         parent.pull(child.branch)
-        # work around fetch bug 212908
-        child.commit('test', rev_id='new-commit2')
-        child.branch.set_last_revision_info(1, 'new-commit')
         child.bzrdir.destroy_workingtree()
         reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
             child.bzrdir)

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2008-04-08 07:44:39 +0000
+++ bzrlib/tests/test_remote.py	2008-05-01 02:15:59 +0000
@@ -829,6 +829,7 @@
         transport_path = 'quack'
         repo, client = self.setup_fake_client_and_repository(
             responses, transport_path)
+        self.assertTrue(client._medium._remote_is_at_least_1_2)
         rev_id = 'revision-id'
         expected_deprecations = [
             'bzrlib.remote.RemoteRepository.get_revision_graph was deprecated '
@@ -842,6 +843,36 @@
              ('call_expecting_body', 'Repository.get_revision_graph',
               ('quack/', ''))],
             client._calls)
+        # The medium is now marked as being connected to an older server
+        self.assertFalse(client._medium._remote_is_at_least_1_2)
+
+    def test_get_parent_map_fallback_parentless_node(self):
+        """get_parent_map falls back to get_revision_graph on old servers.  The
+        results from get_revision_graph are tweaked to match the get_parent_map
+        API.
+
+        Specifically, a {key: ()} result from get_revision_graph means "no
+        parents" for that key, which in get_parent_map results should be
+        represented as {key: ('null:',)}.
+
+        This is the test for https://bugs.launchpad.net/bzr/+bug/214894
+        """
+        rev_id = 'revision-id'
+        responses = [(('ok',), rev_id)]
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(
+            responses, transport_path)
+        client._medium._remote_is_at_least_1_2 = False
+        expected_deprecations = [
+            'bzrlib.remote.RemoteRepository.get_revision_graph was deprecated '
+            'in version 1.4.']
+        parents = self.callDeprecated(
+            expected_deprecations, repo.get_parent_map, [rev_id])
+        self.assertEqual(
+            [('call_expecting_body', 'Repository.get_revision_graph',
+             ('quack/', ''))],
+            client._calls)
+        self.assertEqual({rev_id: ('null:',)}, parents)
 
     def test_get_parent_map_unexpected_response(self):
         responses = [

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2008-04-11 23:05:02 +0000
+++ bzrlib/tests/test_selftest.py	2008-04-12 18:11:07 +0000
@@ -502,7 +502,9 @@
         adapter = TreeTestProviderAdapter(server1, server2, formats)
         suite = adapter.adapt(input_test)
         tests = list(iter(suite))
-        self.assertEqual(4, len(tests))
+        # XXX We should not have tests fail as we add more scenarios
+        # abentley 20080412
+        self.assertEqual(5, len(tests))
         # this must match the default format setp up in
         # TreeTestProviderAdapter.adapt
         default_format = WorkingTreeFormat3

=== modified file 'bzrlib/tests/test_smart.py'
--- bzrlib/tests/test_smart.py	2008-04-08 08:13:33 +0000
+++ bzrlib/tests/test_smart.py	2008-04-29 13:43:40 +0000
@@ -104,7 +104,10 @@
         # the default or a parameterized class, but rather use the
         # TestCaseWithTransport infrastructure to set up a smart server and
         # transport.
-        self.transport_server = smart.server.SmartTCPServer_for_testing
+        self.transport_server = self.make_transport_server
+
+    def make_transport_server(self):
+        return smart.server.SmartTCPServer_for_testing('-' + self.id())
 
     def get_smart_medium(self):
         """Get a smart medium to use in tests."""

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2008-04-14 08:07:08 +0000
+++ bzrlib/tests/test_smart_transport.py	2008-04-20 07:48:29 +0000
@@ -1130,8 +1130,8 @@
     def test_server_started_hook_memory(self):
         """The server_started hook fires when the server is started."""
         self.hook_calls = []
-        server.SmartTCPServer.hooks.install_hook('server_started',
-            self.capture_server_call)
+        server.SmartTCPServer.hooks.install_named_hook('server_started',
+            self.capture_server_call, None)
         self.setUpServer()
         # at this point, the server will be starting a thread up.
         # there is no indicator at the moment, so bodge it by doing a request.
@@ -1144,8 +1144,8 @@
     def test_server_started_hook_file(self):
         """The server_started hook fires when the server is started."""
         self.hook_calls = []
-        server.SmartTCPServer.hooks.install_hook('server_started',
-            self.capture_server_call)
+        server.SmartTCPServer.hooks.install_named_hook('server_started',
+            self.capture_server_call, None)
         self.setUpServer(backing_transport=get_transport("."))
         # at this point, the server will be starting a thread up.
         # there is no indicator at the moment, so bodge it by doing a request.
@@ -1160,8 +1160,8 @@
     def test_server_stopped_hook_simple_memory(self):
         """The server_stopped hook fires when the server is stopped."""
         self.hook_calls = []
-        server.SmartTCPServer.hooks.install_hook('server_stopped',
-            self.capture_server_call)
+        server.SmartTCPServer.hooks.install_named_hook('server_stopped',
+            self.capture_server_call, None)
         self.setUpServer()
         result = [([self.backing_transport.base], self.transport.base)]
         # check the stopping message isn't emitted up front.
@@ -1177,8 +1177,8 @@
     def test_server_stopped_hook_simple_file(self):
         """The server_stopped hook fires when the server is stopped."""
         self.hook_calls = []
-        server.SmartTCPServer.hooks.install_hook('server_stopped',
-            self.capture_server_call)
+        server.SmartTCPServer.hooks.install_named_hook('server_stopped',
+            self.capture_server_call, None)
         self.setUpServer(backing_transport=get_transport("."))
         result = [(
             [self.backing_transport.base, self.backing_transport.external_url()]

=== modified file 'bzrlib/tests/test_store.py'
--- bzrlib/tests/test_store.py	2008-04-08 00:57:07 +0000
+++ bzrlib/tests/test_store.py	2008-04-28 03:52:09 +0000
@@ -433,4 +433,3 @@
         self._transaction = transactions.ReadOnlyTransaction()
         vf = self.vfstore.get_weave_or_empty('id', self._transaction)
         self.assertRaises(errors.ReadOnlyError, vf.add_lines, 'b', [], [])
-

=== modified file 'bzrlib/tests/transport_util.py'
--- bzrlib/tests/transport_util.py	2007-09-24 15:01:26 +0000
+++ bzrlib/tests/transport_util.py	2008-03-10 13:26:34 +0000
@@ -115,8 +115,8 @@
         return url
 
     def start_logging_connections(self):
-        ConnectionHookedTransport.hooks.install_hook('_set_connection',
-                                                     self._collect_connection)
+        ConnectionHookedTransport.hooks.install_named_hook(
+            '_set_connection', self._collect_connection, None)
         # uninstall our hooks when we are finished
         self.addCleanup(self.reset_hooks)
 

=== modified file 'bzrlib/tests/tree_implementations/__init__.py'
--- bzrlib/tests/tree_implementations/__init__.py	2008-04-12 06:03:29 +0000
+++ bzrlib/tests/tree_implementations/__init__.py	2008-04-28 03:02:42 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006, 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@
     WorkingTreeTestProviderAdapter,
     )
 from bzrlib.revisiontree import RevisionTree
+from bzrlib.transform import TransformPreview
 from bzrlib.workingtree import (
     WorkingTreeFormat,
     WorkingTreeFormat3,
@@ -72,6 +73,12 @@
     return tree.basis_tree()
 
 
+def preview_tree_pre(testcase, tree):
+    tt = TransformPreview(tree)
+    testcase.addCleanup(tt.finalize)
+    return tt.get_preview_tree()
+
+
 class TestTreeImplementationSupport(TestCaseWithTransport):
 
     def test_revision_tree_from_workingtree(self):
@@ -314,6 +321,8 @@
         self.scenarios.append(self.create_tree_scenario(
             DirStateRevisionTree.__name__, _dirstate_tree_from_workingtree,
             WorkingTreeFormat4()))
+        self.scenarios.append(self.create_tree_scenario('PreviewTree',
+            preview_tree_pre))
 
     def create_tree_scenario(self, name, converter, workingtree_format=None):
         """Create a scenario for the specified converter

=== modified file 'bzrlib/tests/tree_implementations/test_inv.py'
--- bzrlib/tests/tree_implementations/test_inv.py	2007-12-18 15:22:47 +0000
+++ bzrlib/tests/tree_implementations/test_inv.py	2008-04-28 03:02:42 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2008 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,9 +26,14 @@
 from bzrlib.symbol_versioning import zero_ninetyone, one_zero
 from bzrlib.tests import SymlinkFeature, TestSkipped
 from bzrlib.tests.tree_implementations import TestCaseWithTree
+from bzrlib.transform import _PreviewTree
 from bzrlib.uncommit import uncommit
 
 
+def get_entry(tree, file_id):
+    return tree.iter_entries_by_dir([file_id]).next()[1]
+
+
 class TestEntryDiffing(TestCaseWithTree):
 
     def setUp(self):
@@ -55,12 +60,11 @@
         self.tree_2 = self.workingtree_to_test_tree(self.wt)
         self.tree_2.lock_read()
         self.addCleanup(self.tree_2.unlock)
-        self.inv_2 = self.tree_2.inventory
-        self.file_2 = self.inv_2['fileid']
-        self.file_2b = self.inv_2['binfileid']
+        self.file_2 = get_entry(self.tree_2, 'fileid')
+        self.file_2b = get_entry(self.tree_2, 'binfileid')
         if has_symlinks():
             self.link_1 = self.inv_1['linkid']
-            self.link_2 = self.inv_2['linkid']
+            self.link_2 = get_entry(self.tree_2, 'linkid')
 
     def test_file_diff_deleted(self):
         output = StringIO()
@@ -104,7 +108,7 @@
                                             "-foo\n"
                                             "+bar\n"
                                             "\n")
-        
+
     def test_file_diff_binary(self):
         output = StringIO()
         self.applyDeprecated(one_zero,
@@ -113,7 +117,7 @@
                              "/dev/null", self.tree_1,
                              "new_label", self.file_2b, self.tree_2,
                              output)
-        self.assertEqual(output.getvalue(), 
+        self.assertEqual(output.getvalue(),
                          "Binary files /dev/null and new_label differ\n")
 
     def test_link_diff_deleted(self):
@@ -182,17 +186,17 @@
         self.tree = self.workingtree_to_test_tree(self.wt)
         self.tree.lock_read()
         self.addCleanup(self.tree.unlock)
-        self.file_active = self.tree.inventory['fileid']
+        self.file_active = get_entry(self.tree, 'fileid')
         self.weave = self.branch.repository.weave_store.get_weave('fileid',
             self.branch.repository.get_transaction())
-        
+
     def get_previous_heads(self, inventories):
         return self.applyDeprecated(zero_ninetyone,
             self.file_active.find_previous_heads,
             inventories,
             self.branch.repository.weave_store,
             self.branch.repository.get_transaction())
-        
+
     def test_fileid_in_no_inventory(self):
         self.assertEqual({}, self.get_previous_heads([self.inv_A]))
 
@@ -218,7 +222,7 @@
         self.assertEqual({'D':self.inv_D['fileid']},
                          self.get_previous_heads([self.inv_D, self.inv_B]))
 
-    # TODO: test two inventories with the same file revision 
+    # TODO: test two inventories with the same file revision
 
 
 class TestInventory(TestCaseWithTree):
@@ -227,21 +231,32 @@
         self.tree = self.get_tree_with_subdirs_and_all_content_types()
         self.tree.lock_read()
         self.addCleanup(self.tree.unlock)
-        # Commenting out the following line still fails.
-        self.inv = self.tree.inventory
 
     def test_symlink_target(self):
         self.requireFeature(SymlinkFeature)
         self._set_up()
-        if isinstance(self.tree, MutableTree):
+        if isinstance(self.tree, (MutableTree, _PreviewTree)):
             raise TestSkipped(
-                'symlinks not accurately represented in working trees')
-        entry = self.inv[self.inv.path2id('symlink')]
+                'symlinks not accurately represented in working trees and'
+                ' preview trees')
+        entry = get_entry(self.tree, self.tree.path2id('symlink'))
         self.assertEqual(entry.symlink_target, 'link-target')
 
+    def test_symlink_target_tree(self):
+        self.requireFeature(SymlinkFeature)
+        self._set_up()
+        self.assertEqual('link-target',
+                         self.tree.get_symlink_target('symlink'))
+
+    def test_kind_symlink(self):
+        self.requireFeature(SymlinkFeature)
+        self._set_up()
+        self.assertEqual('symlink', self.tree.kind('symlink'))
+        self.assertIs(None, self.tree.get_file_size('symlink'))
+
     def test_symlink(self):
         self.requireFeature(SymlinkFeature)
         self._set_up()
-        entry = self.inv[self.inv.path2id('symlink')]
+        entry = get_entry(self.tree, self.tree.path2id('symlink'))
         self.assertEqual(entry.kind, 'symlink')
         self.assertEqual(None, entry.text_size)

=== modified file 'bzrlib/tests/tree_implementations/test_test_trees.py'
--- bzrlib/tests/tree_implementations/test_test_trees.py	2007-11-01 09:52:45 +0000
+++ bzrlib/tests/tree_implementations/test_test_trees.py	2008-04-12 19:43:13 +0000
@@ -51,7 +51,7 @@
             [('', 'root-id'), ('a', 'a-id'), ('b', 'b-id'), ('b/c', 'c-id')],
             [(path, node.file_id) for path, node in tree.iter_entries_by_dir()])
         self.assertEqualDiff('contents of a\n', tree.get_file_text('a-id'))
-        self.assertFalse(tree.is_executable('c-id'))
+        self.assertFalse(tree.is_executable('c-id', path='b/c'))
 
     def test_abc_tree_content_2_no_parents(self):
         tree = self.make_branch_and_tree('.')

=== modified file 'bzrlib/tests/workingtree_implementations/test_commit.py'
--- bzrlib/tests/workingtree_implementations/test_commit.py	2008-04-05 02:29:11 +0000
+++ bzrlib/tests/workingtree_implementations/test_commit.py	2008-04-20 08:21:39 +0000
@@ -498,8 +498,8 @@
         ui.ui_factory = factory
         def a_hook(_, _2, _3, _4, _5, _6):
             pass
-        branch.Branch.hooks.install_hook('post_commit', a_hook)
-        branch.Branch.hooks.name_hook(a_hook, 'hook name')
+        branch.Branch.hooks.install_named_hook('post_commit', a_hook,
+                                               'hook name')
         tree.commit('first post')
         self.assertEqual(
             [('update', 1, 5, 'Collecting changes [Directory 0] - Stage'),
@@ -523,8 +523,8 @@
         ui.ui_factory = factory
         def a_hook(_, _2, _3, _4, _5, _6, _7, _8):
             pass
-        branch.Branch.hooks.install_hook('pre_commit', a_hook)
-        branch.Branch.hooks.name_hook(a_hook, 'hook name')
+        branch.Branch.hooks.install_named_hook('pre_commit', a_hook,
+                                               'hook name')
         tree.commit('first post')
         self.assertEqual(
             [('update', 1, 5, 'Collecting changes [Directory 0] - Stage'),
@@ -548,8 +548,10 @@
             mutabletree.MutableTree.hooks['start_commit'] = []
         self.addCleanup(restoreDefaults)
         tree = self.make_branch_and_tree('.')
-        mutabletree.MutableTree.hooks.install_hook('start_commit', 
-            start_commit_hook_adds_file)
+        mutabletree.MutableTree.hooks.install_named_hook(
+            'start_commit',
+            start_commit_hook_adds_file,
+            None)
         revid = tree.commit('first post')
         committed_tree = tree.basis_tree()
         self.assertTrue(committed_tree.has_filename("newfile"))

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2008-04-04 04:20:19 +0000
+++ bzrlib/transform.py	2008-04-28 18:50:55 +0000
@@ -25,7 +25,8 @@
     bzrdir,
     delta,
     errors,
-    inventory
+    inventory,
+    revision as _mod_revision,
     )
 """)
 from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
@@ -1359,11 +1360,33 @@
             yield self.trans_id_tree_path(childpath)
 
 
-class _PreviewTree(object):
+class _PreviewTree(tree.Tree):
     """Partial implementation of Tree to support show_diff_trees"""
 
     def __init__(self, transform):
         self._transform = transform
+        self._final_paths = FinalPaths(transform)
+
+    def _changes(self, file_id):
+        for changes in self._transform.iter_changes():
+            if changes[0] == file_id:
+                return changes
+
+    def _content_change(self, file_id):
+        """Return True if the content of this file changed"""
+        changes = self._changes(file_id)
+        # changes[2] is true if the file content changed.  See
+        # InterTree.iter_changes.
+        return (changes is not None and changes[2])
+
+    def _get_file_revision(self, file_id, vf, tree_revision):
+        return self._transform._tree._get_file_revision(file_id, vf,
+                                                        tree_revision)
+
+    def _stat_limbo_file(self, file_id):
+        trans_id = self._transform.trans_id_file_id(file_id)
+        name = self._transform._limbo_name(trans_id)
+        return os.lstat(name)
 
     def lock_read(self):
         # Perhaps in theory, this should lock the TreeTransform?
@@ -1372,6 +1395,75 @@
     def unlock(self):
         pass
 
+    @property
+    def inventory(self):
+        """This Tree does not use inventory as its backing data."""
+        raise NotImplementedError(_PreviewTree.inventory)
+
+    def get_root_id(self):
+        return self._transform.final_file_id(self._transform.root)
+
+    def all_file_ids(self):
+        return self._transform._tree.all_file_ids()
+
+    def __iter__(self):
+        return iter(self.all_file_ids())
+
+    def paths2ids(self, specific_files, trees=None, require_versioned=False):
+        """See Tree.paths2ids"""
+        to_find = set(specific_files)
+        result = set()
+        for (file_id, paths, changed, versioned, parent, name, kind,
+             executable) in self._transform.iter_changes():
+            if paths[1] in to_find:
+                result.append(file_id)
+                to_find.remove(paths[1])
+        result.update(self._transform._tree.paths2ids(to_find,
+                      trees=[], require_versioned=require_versioned))
+        return result
+
+    def path2id(self, path):
+        return self._transform._tree.path2id(path)
+
+    def id2path(self, file_id):
+        trans_id = self._transform.trans_id_file_id(file_id)
+        try:
+            return self._final_paths._determine_path(trans_id)
+        except NoFinalPath:
+            raise errors.NoSuchId(self, file_id)
+
+    def iter_entries_by_dir(self, specific_file_ids=None):
+        return self._transform._tree.iter_entries_by_dir(specific_file_ids)
+
+    def kind(self, file_id):
+        trans_id = self._transform.trans_id_file_id(file_id)
+        return self._transform.final_kind(trans_id)
+
+    def stored_kind(self, file_id):
+        return self._transform._tree.stored_kind(file_id)
+
+    def get_file_mtime(self, file_id, path=None):
+        """See Tree.get_file_mtime"""
+        if not self._content_change(file_id):
+            return self._transform._tree.get_file_mtime(file_id, path)
+        return self._stat_limbo_file(file_id).st_mtime
+
+    def get_file_size(self, file_id):
+        """See Tree.get_file_size"""
+        if self.kind(file_id) == 'file':
+            return self._transform._tree.get_file_size(file_id)
+        else:
+            return None
+
+    def get_file_sha1(self, file_id, path=None, stat_value=None):
+        return self._transform._tree.get_file_sha1(file_id)
+
+    def is_executable(self, file_id, path=None):
+        return self._transform._tree.is_executable(file_id, path)
+
+    def path_content_summary(self, path):
+        return self._transform._tree.path_content_summary(path)
+
     def iter_changes(self, from_tree, include_unchanged=False,
                       specific_files=None, pb=None, extra_trees=None,
                       require_versioned=True, want_unversioned=False):
@@ -1391,31 +1483,45 @@
             raise ValueError('want_unversioned is not supported')
         return self._transform.iter_changes()
 
-    def kind(self, file_id):
-        trans_id = self._transform.trans_id_file_id(file_id)
-        return self._transform.final_kind(trans_id)
-
-    def get_file_mtime(self, file_id, path=None):
-        """See Tree.get_file_mtime"""
-        trans_id = self._transform.trans_id_file_id(file_id)
-        name = self._transform._limbo_name(trans_id)
-        return os.stat(name).st_mtime
-
-    def get_file(self, file_id):
+    def get_file(self, file_id, path=None):
         """See Tree.get_file"""
+        if not self._content_change(file_id):
+            return self._transform._tree.get_file(file_id, path)
         trans_id = self._transform.trans_id_file_id(file_id)
         name = self._transform._limbo_name(trans_id)
         return open(name, 'rb')
 
+    def get_file_text(self, file_id):
+        text_file = self.get_file(file_id)
+        try:
+            return text_file.read()
+        finally:
+            text_file.close()
+
+    def annotate_iter(self, file_id,
+                      default_revision=_mod_revision.CURRENT_REVISION):
+        return self._transform._tree.annotate_iter(file_id,
+            default_revision=default_revision)
+
     def get_symlink_target(self, file_id):
         """See Tree.get_symlink_target"""
+        if not self._content_change(file_id):
+            return self._transform._tree.get_symlink_target(file_id)
         trans_id = self._transform.trans_id_file_id(file_id)
         name = self._transform._limbo_name(trans_id)
         return os.readlink(name)
 
-    def paths2ids(self, specific_files, trees=None, require_versioned=False):
-        """See Tree.paths2ids"""
-        return 'not_empty'
+    def list_files(self, include_root=False):
+        return self._transform._tree.list_files(include_root)
+
+    def walkdirs(self, prefix=""):
+        return self._transform._tree.walkdirs(prefix)
+
+    def get_parent_ids(self):
+        return self._transform._tree.get_parent_ids()
+
+    def get_revision_tree(self, revision_id):
+        return self._transform._tree.get_revision_tree(revision_id)
 
 
 def joinpath(parent, child):

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2008-03-14 16:32:01 +0000
+++ bzrlib/tree.py	2008-04-28 16:37:29 +0000
@@ -254,6 +254,15 @@
         """
         raise NotImplementedError(self.get_file_mtime)
 
+    def get_file_size(self, file_id):
+        """Return the size of a file in bytes.
+
+        This applies only to regular files.  If invoked on directories or
+        symlinks, it will return None.
+        :param file_id: The file-id of the file
+        """
+        raise NotImplementedError(self.get_file_size)
+
     def get_file_by_path(self, path):
         return self.get_file(self._inventory.path2id(path), path)
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2008-03-15 14:07:55 +0000
+++ bzrlib/workingtree.py	2008-04-28 16:37:29 +0000
@@ -618,7 +618,14 @@
     __contains__ = has_id
 
     def get_file_size(self, file_id):
-        return os.path.getsize(self.id2abspath(file_id))
+        """See Tree.get_file_size"""
+        try:
+            return os.path.getsize(self.id2abspath(file_id))
+        except OSError, e:
+            if e.errno != errno.ENOENT:
+                raise
+            else:
+                return None
 
     @needs_read_lock
     def get_file_sha1(self, file_id, path=None, stat_value=None):

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2008-04-09 00:34:54 +0000
+++ bzrlib/workingtree_4.py	2008-04-28 16:37:29 +0000
@@ -1570,6 +1570,7 @@
         return self._get_weave(file_id).get_lines(entry[1][4])
 
     def get_file_size(self, file_id):
+        """See Tree.get_file_size"""
         return self.inventory[file_id].text_size
 
     def get_file_text(self, file_id):

=== modified file 'doc/default.css'
--- doc/default.css	2007-08-07 14:56:50 +0000
+++ doc/default.css	2008-02-07 07:05:13 +0000
@@ -35,6 +35,21 @@
   color: inherit;
   }
 
+/* Format ".. note:" sections nicely */
+div.note {
+  margin-left: 5em;
+  margin-right: 5em;
+  color: #000000;
+  background-color: #c1d1ff;
+  border: 1px solid #888888;
+  padding-left: 1em;
+  padding-right: 1em;
+  }
+
+div.note .first {
+  font-weight: bold;
+  }
+
 h1 {
   color: #b52b2b; 
   /* DKREDcolor: #966b72; */
@@ -124,7 +139,6 @@
   margin-right: 5em;
   color: #000000;
   font-weight: normal;
-  background-color: #c1d1ff;
   background-color: #e5ecf9;
   border: 1px solid #888888;
   padding: 1em;

=== modified file 'doc/developers/HACKING.txt'
--- doc/developers/HACKING.txt	2008-05-02 02:05:46 +0000
+++ doc/developers/HACKING.txt	2008-05-02 02:31:14 +0000
@@ -1758,6 +1758,5 @@
   fixing them. Nice.
 
 
-
 ..
    vim: ft=rst tw=74 ai

=== modified file 'doc/developers/network-protocol.txt'
--- doc/developers/network-protocol.txt	2007-11-09 20:49:54 +0000
+++ doc/developers/network-protocol.txt	2008-05-01 00:43:34 +0000
@@ -118,7 +118,7 @@
 
   REQUEST := MESSAGE_V1
   RESPONSE := MESSAGE_V1
-  MESSAGE_V1 := ARGS BODY
+  MESSAGE_V1 := ARGS [BODY]
 
   ARGS := ARG [MORE_ARGS] NEWLINE
   MORE_ARGS := SEP ARG [MORE_ARGS]
@@ -144,7 +144,8 @@
 
 The response protocol is::
 
-  RESPONSE_V2 := "bzr response 2" NEWLINE MESSAGE_V2
+  RESPONSE_V2 := "bzr response 2" NEWLINE RESPONSE_STATUS NEWLINE MESSAGE_V2
+  RESPONSE_STATUS := "success" | "failed"
 
 Future versions should follow this structure, like version two does::
 
@@ -160,7 +161,7 @@
 
 Version two of the message protocol is::
 
-  MESSAGE_V2 := ARGS BODY
+  MESSAGE_V2 := ARGS [BODY_V2]
   BODY_V2 := BODY | STREAMED_BODY
 
 That is, a version one length-prefixed body, or a version two streamed
@@ -174,8 +175,8 @@
  
   STREAMED_BODY := "chunked" NEWLINE CHUNKS TERMINATOR
   CHUNKS := CHUNK [CHUNKS]
-  CHUNK := CHUNK_LENGTH CHUNK_CONTENT
-  CHUNK_LENGTH := HEX_DIGITS NEWLINE
+  CHUNK := HEX_LENGTH CHUNK_CONTENT
+  HEX_LENGTH := HEX_DIGITS NEWLINE
   CHUNK_CONTENT := bytes
   
   TERMINATOR := SUCCESS_TERMINATOR | ERROR_TERMINATOR
@@ -196,6 +197,107 @@
 same for a given request method.  Only new request methods introduced in
 Bazaar 0.91 and later use streamed bodies.
 
+Version three
+-------------
+
+.. note::
+  
+  For some discussion of the requirements that led to this new protocol
+  version, see `bug #83935`_.
+
+.. _bug #83935: https://bugs.launchpad.net/bzr/+bug/83935
+
+Version three has bencoding of most protocol structures, to make parsing
+simpler.  For extra parsing convenience, these structures are length
+prefixed::
+
+  LENGTH_PREFIX := 32-bit unsigned integer in network byte order
+
+Unlike earlier versions, clients and servers are no longer required to
+know which request verbs and responses will have bodies attached.  Because
+of length-prefixing and other changes, it is always possible to know when
+a complete request or response has been read, even if the server
+implements no verbs.
+
+The underlying message format is::
+
+  MESSAGE := "bzr message 3" NEWLINE HEADERS MESSAGE_PARTS
+  HEADERS := LENGTH_PREFIX bencoded_dict
+  MESSAGE_PARTS := MESSAGE_PART [MORE_MESSAGE_PARTS]
+  MORE_MESSAGE_PARTS := END_MESSAGE_PARTS | MESSAGE_PARTS
+  END_MESSAGE_PARTS := "e"
+
+  MESSAGE_PART := ONE_BYTE | STRUCTURE | BYTES
+  ONE_BYTE := "o" byte
+  STRUCTURE := "s" LENGTH_PREFIX bencoded_structure
+  BYTES := "b" LENGTH_PREFIX bytes
+
+This format allows an arbitrary sequence of message parts to be encoded
+in a single message.
+
+Headers
+~~~~~~~
+
+Each request and response will have “headers”, a dictionary of key-value pairs.
+The keys must be strings, not any other type of value.
+
+Currently, the only defined header is “Software version”.  Both the client and
+the server should include a “Software version” header, with a value of a
+free-form string such as “bzrlib 1.5”, to aid debugging and logging.  Clients
+and servers **should not** vary behaviour based on this string.
+
+Conventional requests and responses
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+By convention, most requests and responses have a simple “arguments plus
+optional body” structure, as in earlier protocol versions.  This section
+describes how such messages are encoded.  All requests and responses
+defined by earlier protocol versions must be encoded in this way.
+
+Conventional requests will send a sequence of:
+
+* Arguments (a STRUCTURE of a tuple)
+
+* (Optional) body
+
+  * Single body (BYTES), or
+
+  * Streamed body (multiple BYTES parts), followed by a status (ONE_BYTE)
+
+    * if status is "E", followed by an Error (STRUCTURE)
+
+Conventional responses will send a sequence of:
+
+* Status (ONE_BYTE)
+
+* Arguments (a STRUCTURE of a tuple)
+
+* (Optional) body
+
+  * Single body (BYTES), or
+
+  * Streamed body (multiple BYTES parts), followed by a status (ONE_BYTE)
+
+    * if status is "E", followed by an Error (STRUCTURE)
+
+In all cases, the ONE_BYTE status is either "S" for Success or "E" for
+Error.  Note that the streamed body from version two is now just multiple
+BYTES parts.
+
+For new methods, these sequences are just a convention and may be varied
+if appropriate for a particular request or response.  However, each
+request should at least start with a STRUCTURE encoding the arguments
+tuple.  The first element of that tuple must be a string that names the
+request method.  (Note that arguments in this protocol version are
+bencoded.  As a result, unlike previous protocol versions, arguments in
+this version are 8-bit clean.)
+
+For errors (where the Status byte of a response or a streamed body is
+"E"), the situation is analagous to requests.  The first item in the
+encoded sequence must be a string of the error name.  The other arguments
+supply details about the error, and their number and types will depend on
+the type of error (as identified by the error name).
+
 Paths
 =====
 
@@ -230,6 +332,18 @@
 Contributions welcome!
 
 
+Recognised errors
+=================
+
+The first argument of an error response specifies the error type.
+
+One possible error name is ``UnknownMethod``, which means the server does
+not recognise the verb used by the client's request.  This error was
+introduced in version three.
+
+**XXX**: ideally the error types should be documented here.  Contributions
+welcome!
+
 ..
    vim: ft=rst tw=74 ai
 

=== modified file 'doc/developers/releasing.txt'
--- doc/developers/releasing.txt	2008-05-02 02:14:39 +0000
+++ doc/developers/releasing.txt	2008-05-02 02:31:14 +0000
@@ -93,15 +93,11 @@
         if version_info[3] == 'final':
             version_string = '%d.%d.%d' % version_info[:3]
 
-#. Send the changes to PQM, to update the official master branch.
+#. Submit those changes to PQM for merge into the appropriate release
+   branch.
 
 #. When PQM succeeds, pull down the master release branch.
 
-#. Merge the release branch back into the trunk.  Check that changes in NEWS were merged into the right sections.  
-   If it's not already done, advance the version number in ``bzr`` and
-   ``bzrlib/__init__.py``.
-   Submit this back into pqm for bzr.dev.
-
 Making the source tarball
 -------------------------
 
@@ -109,6 +105,9 @@
   
      make dist
 
+#. Unpack the tarball into a temporary directory and run ``make check`` in
+   that directory, to check for packaging problems.
+
 
 Publishing the release
 ----------------------
@@ -189,8 +188,17 @@
    Remember to check the results afterwards.
 
 
-The Bazaar PPA archive
-----------------------
+Merging the released code back to trunk
+---------------------------------------
+
+Merge the release branch back into the trunk.  Check that changes in NEWS
+were merged into the right sections.  If it's not already done, advance
+the version number in ``bzr`` and ``bzrlib/__init__.py``.  Submit this
+back into pqm for bzr.dev.
+
+
+Updating the PPA for a new release
+----------------------------------
 
 We build Ubuntu ``.deb`` packages for Bazaar as an important part of the release
 process.  These packages are hosted in a `Personal Package Archive (PPA)`__ on
@@ -206,10 +214,6 @@
 The ``debian/`` directory containing the packaging information is kept in
 branches on Launchpad, named like 
 <https://code.launchpad.net/~bzr/bzrtools/packaging-dapper>
-
-Updating the PPA for a new release
-----------------------------------
-
 Preconditions for building these packages:
   
 * You must have a Launchpad account and be a member of the `~bzr`__ team

=== modified file 'doc/en/user-guide/hooks.txt'
--- doc/en/user-guide/hooks.txt	2008-03-11 00:45:46 +0000
+++ doc/en/user-guide/hooks.txt	2008-03-15 00:51:43 +0000
@@ -22,8 +22,8 @@
         print "The new revno is %d" % push_result.new_revno
 
 
-    branch.Branch.hooks.install_hook('post_push', post_push_hook)
-    branch.Branch.hooks.name_hook(post_push_hook, 'My post_push hook')
+    branch.Branch.hooks.install_hook('post_push', post_push_hook,
+                                     'My post_push hook')
 
 To use this example, create a file named ``push_hook.py``, and stick it in
 ``plugins`` subdirectory of your configuration directory.  (If you have never
@@ -34,10 +34,9 @@
 single argument, the ``push_result``.
 
 Next, we install the hook.  ``'post_push'`` identifies where we want to install
-the hook, and the second parameter is the hook itself.
-
-Finally, we name the hook.  This is optional, but it means the hook name can
-be used in progress messages and error messages.
+the hook, and the second parameter is the hook itself.  We also give the hook a
+name 'My post_push hook', which can be used in progress messages and error
+messages.
 
 That's it!  The next time you push, it should show "The new revno is...".
 Of course, hooks can be much more elaborate than this, because you have the

